from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class PropertyData(TcBaseObj):
    """
    PropertyData containing synchronization result for the propertyName. This can be used for the result of dry run and
    for displaying the logs.
    
    :var propertyName: Name of the property. Supported values are:
    - status: Specifies the status of the operation. Valid values of status are "Pass", "Fail", "Partial" and "NA" (Not
    Applicable). "Pass" signifies that source and target object synchronization operation is passed, "Fail" signifies
    failed, "Partial" signifies partially passed and "NA" signifies that the operation is not applicable for the source
    and target object.
    - statusDisplayValue: Specifies if the result for synchronization operation is to be displayed or not in UI. Valid
    values are "True" and "False".
    - failedBOMLines: Specifies BOMLines for which synchronization operation is failed. Valid values are uid of the
    BOMLines.
    
    
    :var values: Value of the property. See propertyName for supported values.
    :var dataType: Type of the data. Supported values are: "boolean", "string" and "businessObject".
    """
    propertyName: str = ''
    values: List[str] = ()
    dataType: str = ''


@dataclass
class SyncMasterAndAlternativeInputInfo(TcBaseObj):
    """
    The input to the service is a structure which contains list of objects to be synchronized.
    
    :var syncStructsInfo: A list of SyncStructsInfo containing source and target object(s) to be synchronized.
    :var syncParams: A map (string, string) representing synchronization parameter and its value.
    Valid options are:
    - ("DryRun", "true") for validating the synchronization scenario before the actual updates are done.
    - ("ExcludeExistingStudy", "true") for synchronizing the study objects and to exclude the study objects if already
    present in target structure.
    
    """
    syncStructsInfo: List[SyncStructsInfo] = ()
    syncParams: SyncParams = None


@dataclass
class SyncMasterAndAlternativeResponse(TcBaseObj):
    """
    The response contains the list of synchronization results.
    
    :var serviceData: The service data containing partial errors.
    :var syncResult: A list of SyncResult containing result of syncMasterAndAlternative operation.
    """
    serviceData: ServiceData = None
    syncResult: List[SyncResult] = ()


@dataclass
class SyncResult(TcBaseObj):
    """
    A list of SyncResult containing result of syncMasterAndAlternative operation.
    
    :var source: The source object.
    :var target: The target object.
    :var resultData: A list of PropertyData containing synchronization result for the propertyName. This can be used
    for the result of dry run and for displaying the logs.
    """
    source: BusinessObject = None
    target: BusinessObject = None
    resultData: List[PropertyData] = ()


@dataclass
class SyncStructsInfo(TcBaseObj):
    """
    A list of SyncStructsInfo containing source and target object(s) to be synchronized.
    
    :var target: A list of target objects to be synchronized. It can be BOMLine or MECollaborationContext (CC) objects.
    :var source: A list of source BOMLine objects to be synchronized. It can be Study top line of type Mfg0BvrStudy or
    Operation line of type Mfg0BvrOperation.
    """
    target: List[BusinessObject] = ()
    source: List[BusinessObject] = ()


"""
A map (string, string) representing synchronization parameter and its value.
Valid options are:
- ("DryRun", "true") for validating the synchronization scenario before the actual updates are done.
- ("ExcludeExistingStudy", "true") for synchronizing the study objects and to exclude the study objects if already present in target structure.


"""
SyncParams = Dict[str, str]



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, ImanQuery
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class ExecuteSavedQueryResponse(TcBaseObj):
    """
    Holds the response for ExecuteSavedQuery. The number of objects found is the 
     total number found and may be greater than the number return in the vector of 
     objects if a limit was specified on executing the query.
    
    :var nFound: The number of objects found by the query.
    :var objects: The objects returned.
    :var serviceData: The standard service data which includes the returned objects.
    """
    nFound: int = 0
    objects: List[BusinessObject] = ()
    serviceData: ServiceData = None


@dataclass
class GetSavedQueriesResponse(TcBaseObj):
    """
    Holds the response for GetSavedQueries
    
    :var queries: A list of SavedQueryObjects each of which contains the query, query name, and query description for a
    saved query.
    :var serviceData: Standard ServiceData
    """
    queries: List[SavedQueryObject] = ()
    serviceData: ServiceData = None


@dataclass
class SavedQueryFieldListObject(TcBaseObj):
    """
    A  list of saved query fields for a saved query.
    
    :var fields: Holds one entry per clause
    """
    fields: List[SavedQueryFieldObject] = ()


@dataclass
class SavedQueryFieldObject(TcBaseObj):
    """
    The data for a field of a saved query.
    
    :var attributeName: The attribute name for the clause.
    :var entryName: User entry name for clause.
    :var logicalOperation: Logical operator for clause
    :var mathOperation: Math operator for clause
    :var value: Default value for clause
    :var lov: LOV for the clause
    :var attributeType: Attribute type for clause
    """
    attributeName: str = ''
    entryName: str = ''
    logicalOperation: str = ''
    mathOperation: str = ''
    value: str = ''
    lov: BusinessObject = None
    attributeType: int = 0


@dataclass
class SavedQueryObject(TcBaseObj):
    """
    The data for each saved query found.
    
    :var query: The saved query.
    :var name: The name of the saved query.
    :var description: The description of the saved query.
    """
    query: ImanQuery = None
    name: str = ''
    description: str = ''


@dataclass
class DescribeSavedQueriesResponse(TcBaseObj):
    """
    Holds the field data for each saved query.
    
    :var fieldLists: A list of fields for each input query.
    :var serviceData: The standard ServiceData.
    """
    fieldLists: List[SavedQueryFieldListObject] = ()
    serviceData: ServiceData = None



from __future__ import annotations

from tcsoa.gen.BusinessObjects import Role, EPMTaskTemplate, Signoff, BusinessObject, EPMAssignmentList, EPMTask, Group, ResourcePool, EPMSignoffProfile
from enum import Enum
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class GetResourcePoolOutput(TcBaseObj):
    """
    Structure containing the Resource Pool matching the GroupRoleRef and the ServiceData
    
    :var resourcePoolInfo: resourcePoolInfo
    :var serviceData: serviceData
    """
    resourcePoolInfo: List[ResourcePoolInfo] = ()
    serviceData: ServiceData = None


@dataclass
class AssignmentLists(TcBaseObj):
    """
    Process Assignment Lists
    
    :var assignedLists: assignedLists
    :var ownedLists: ownedLists
    :var groupLists: groupLists
    :var otherLists: otherLists
    :var serviceData: Service data - returns index of the input vector if call on that fails
    """
    assignedLists: List[EPMAssignmentList] = ()
    ownedLists: List[EPMAssignmentList] = ()
    groupLists: List[EPMAssignmentList] = ()
    otherLists: List[EPMAssignmentList] = ()
    serviceData: ServiceData = None


@dataclass
class GroupRoleRef(TcBaseObj):
    """
    Structure containing the group and role info
    
    :var groupTag: groupTag
    :var roleTag: roleTag
    :var allowSubGroup: allowSubGroup
    :var isAllMembers: isAllMembers
    """
    groupTag: Group = None
    roleTag: Role = None
    allowSubGroup: int = 0
    isAllMembers: int = 0


@dataclass
class InstanceInfo(TcBaseObj):
    """
    Structure containing information related to the new process
    
    :var instanceKey: instanceKey. supported in future
    :var newProcessDepTask: newProcessDepTask. supported in future
    :var newProcessUrl: newProcessUrl. supported in future
    :var newProcessDepTaskUrl: newProcessDepTaskUrl. supported in future
    :var serviceData: serviceData containing the created process and errors if any
    """
    instanceKey: str = ''
    newProcessDepTask: str = ''
    newProcessUrl: str = ''
    newProcessDepTaskUrl: str = ''
    serviceData: ServiceData = None


@dataclass
class AttachmentInfo(TcBaseObj):
    """
    Structure containing list of attachments and the attachment types
    
    :var attachment: attachment
    :var attachmentType: attachmentType
    """
    attachment: List[BusinessObject] = ()
    attachmentType: List[int] = ()


@dataclass
class AuditFile(TcBaseObj):
    """
    Audit files
    
    :var auditFiles: auditFiles
    :var serviceData: serviceData
    """
    auditFiles: List[str] = ()
    serviceData: ServiceData = None


@dataclass
class ProcessTemplates(TcBaseObj):
    """
    Process templates
    
    :var output: vector of DefinitionInfo objects
    :var serviceData: Service data contains any partial errors.
    """
    output: List[DefinitionInfo] = ()
    serviceData: ServiceData = None


@dataclass
class RemoveSignoffsInfo(TcBaseObj):
    """
    Structure containing signoffs to be removed
    
    :var task: task
    :var removeSignoffObjs: removeSignoffObjs
    """
    task: EPMTask = None
    removeSignoffObjs: List[Signoff] = ()


@dataclass
class ResourcePoolInfo(TcBaseObj):
    """
    Structure containing resource pool , group and role info
    
    :var groupRoleRef: groupRoleRef
    :var resourcePoolTag: resourcePoolTag
    """
    groupRoleRef: GroupRoleRef = None
    resourcePoolTag: ResourcePool = None


@dataclass
class Resources(TcBaseObj):
    """
    Structure containing the task template and the resources to be applied to the template
    
    :var taskTemplate: taskTemplate
    :var templateResources: templateResources
    :var profiles: profiles
    :var actions: actions
    :var revQuorum: revQuorum
    :var ackQuorum: ackQuorum
    """
    taskTemplate: EPMTaskTemplate = None
    templateResources: List[BusinessObject] = ()
    profiles: List[EPMSignoffProfile] = ()
    actions: List[int] = ()
    revQuorum: int = 0
    ackQuorum: int = 0


@dataclass
class Tasks(TcBaseObj):
    """
    Structure containing all the tasks in a process
    
    :var allTasks: allTasks
    :var serviceData: Service data contains any partial errors.
    """
    allTasks: List[EPMTask] = ()
    serviceData: ServiceData = None


@dataclass
class Templates(TcBaseObj):
    """
    structure containing workflow templates
    
    :var workflowTemplates: workflowTemplates
    :var serviceData: Service data contains any partial errors.
    """
    workflowTemplates: List[EPMTaskTemplate] = ()
    serviceData: ServiceData = None


@dataclass
class ChangeStateInputInfo(TcBaseObj):
    """
    Input to changeState operation
    
    :var state: state
    :var remoteProcessID: remoteProcessID
    """
    state: stateType = None
    remoteProcessID: str = ''


@dataclass
class ChangeStateOutput(TcBaseObj):
    """
    ChangeState operation output
    
    :var state: state
    :var remoteProcessID: remoteProcessID
    :var parentProcessID: parentProcessID
    :var serviceData: serviceData
    """
    state: stateType = None
    remoteProcessID: str = ''
    parentProcessID: str = ''
    serviceData: ServiceData = None


@dataclass
class ContextData(TcBaseObj):
    """
    Context-specific data required to create a process or a sub process at a local or a remote site.  Except the
    process template, the rest of the elements in the context data is optional. The optional elements        that are
    currently supported are attachmentCount,attachments,attachmentTypes and processAssignmentList.  The rest of the
    optional elements are defined to support        future workflow enhancements.
    
    :var processTemplate: Name of the process template.  must be a valid, existing process template
    :var processOwner: Login id of the process owner. supported in future
    :var dependencyTask: The path to the dependency task. This will be used to determine the dependency tasks in the
    processes that need to be returned in response.   The full path to the task in the template needs to be specified,
    e.g.       CMII Change Notice: Change Admin II (CN). supported in future
    :var subscribeToEvents: The flag to indicate if the observer needs to be notified about events that occur on the
    processes and the dependency tasks in the processes. supported in future
    :var subscriptionEventCount: Number of events to subscribe to
    :var subscriptionEventList: List of events for which subscription objects will be created so that the observer is
    notified when these events occur on the process and /or process dependency task. supported in future
    :var remoteParent: URI of the observer in the remote application. supported in future
    :var remoteParentAppguid: The Application ID of the application in which the observer resides. supported in future
    :var remoteParentUrl: The URL to the observer. supported in future
    :var attachmentCount: Count of attachment objects consisting of including both target        and reference
    attachments. If count is less than 1, no attachments are added
    :var attachments: List of atachments representing either target or reference objects that will be added at process
    creation time. List may consist of target attachments or reference attachments or a mixture of both. If NULL, no
    attachments are added
    :var attachmentTypes: Identifies the types of attachments listed in attachment.  Valid types include
    EPM_target_attachment (target attachment) and EPM_reference_attachment (reference attachment). There is a
    one-to-one correspondence between the attachment types on this list and the list of attachments.
    :var deadlineDate: A date in the form of yyyy-mm-dd hh-mm-ss GMT that will be applied as a due date for the
    processes. If GMT is not specified then the time will be interpreted as based on the local time zone of the server.
    If NULL or invalid date, no due date is applied. supported in future
    :var container: Identifies the object to which the processes should be attached using the relation_type specified.
    supported in future
    :var relationType: The name of the relation. If NULL, the default relation type for the container object will be
    used. supported in future
    :var processAssignmentList: Name of the process assignment list
    :var processResources: A list of comma-separated user login IDs that will be used to satisfy signoff profiles for
    each individual task. Any users that do not match a signoff profile within a task will be added as an adhoc user.
    supported in future
    """
    processTemplate: str = ''
    processOwner: str = ''
    dependencyTask: str = ''
    subscribeToEvents: bool = False
    subscriptionEventCount: int = 0
    subscriptionEventList: List[str] = ()
    remoteParent: str = ''
    remoteParentAppguid: str = ''
    remoteParentUrl: str = ''
    attachmentCount: int = 0
    attachments: List[str] = ()
    attachmentTypes: List[int] = ()
    deadlineDate: datetime = None
    container: str = ''
    relationType: str = ''
    processAssignmentList: str = ''
    processResources: List[str] = ()


@dataclass
class CreateSignoffInfo(TcBaseObj):
    """
    Structure containing information to add adhoc signoff or add signoff based on a profile
    
    :var signoffMember: signoffMember
    :var origin: origin
    :var signoffAction: signoffAction
    :var originType: originType
    """
    signoffMember: BusinessObject = None
    origin: BusinessObject = None
    signoffAction: SignoffAction = None
    originType: OriginType = None


@dataclass
class CreateSignoffs(TcBaseObj):
    """
    Structure containing information to add adhoc signoff or add signoff based on a profile
    
    :var task: task
    :var signoffInfo: signoffInfo
    """
    task: EPMTask = None
    signoffInfo: List[CreateSignoffInfo] = ()


@dataclass
class DefinitionInfo(TcBaseObj):
    """
    Structure containing process template information
    
    :var definitionkey: unique identifier of the process template
    :var name: name of the process template
    :var description: description of the process template
    :var version: version of the process template
    :var status: status of the process template. The values are 0 -obsolete,1 - under construction, 2- ready for use
    """
    definitionkey: str = ''
    name: str = ''
    description: str = ''
    version: str = ''
    status: int = 0


class AllOrAssigned(Enum):
    """
    AllOrAssigned
    """
    SOA_EPM_All = 'SOA_EPM_All'
    SOA_EPM_Assigned = 'SOA_EPM_Assigned'


class OriginType(Enum):
    """
    OriginType
    """
    SOA_EPM_ORIGIN_UNDEFINED = 'SOA_EPM_ORIGIN_UNDEFINED'
    SOA_EPM_SIGNOFF_ORIGIN_PROFILE = 'SOA_EPM_SIGNOFF_ORIGIN_PROFILE'
    SOA_EPM_SIGNOFF_ORIGIN_ADDRESSLIST = 'SOA_EPM_SIGNOFF_ORIGIN_ADDRESSLIST'


class SignoffAction(Enum):
    """
    SignoffAction
    """
    SOA_EPM_ACTION_UNDEFINED = 'SOA_EPM_ACTION_UNDEFINED'
    SOA_EPM_Review = 'SOA_EPM_Review'
    SOA_EPM_Acknowledge = 'SOA_EPM_Acknowledge'
    SOA_EPM_Notify = 'SOA_EPM_Notify'


class SoaEPMAction(Enum):
    """
    SoaEPMAction
    """
    SOA_EPM_assign_action = 'SOA_EPM_assign_action'
    SOA_EPM_start_action = 'SOA_EPM_start_action'
    SOA_EPM_remove_attachment_action = 'SOA_EPM_remove_attachment_action'
    SOA_EPM_approve_action = 'SOA_EPM_approve_action'
    SOA_EPM_reject_action = 'SOA_EPM_reject_action'
    SOA_EPM_promote_action = 'SOA_EPM_promote_action'
    SOA_EPM_demote_action = 'SOA_EPM_demote_action'
    SOA_EPM_refuse_action = 'SOA_EPM_refuse_action'
    SOA_EPM_assign_approver_action = 'SOA_EPM_assign_approver_action'
    SOA_EPM_notify_action = 'SOA_EPM_notify_action'
    SOA_EPM_no_action = 'SOA_EPM_no_action'
    SOA_EPM_fail_action = 'SOA_EPM_fail_action'
    SOA_EPM_complete_action = 'SOA_EPM_complete_action'
    SOA_EPM_skip_action = 'SOA_EPM_skip_action'
    SOA_EPM_suspend_action = 'SOA_EPM_suspend_action'
    SOA_EPM_resume_action = 'SOA_EPM_resume_action'
    SOA_EPM_undo_action = 'SOA_EPM_undo_action'
    SOA_EPM_abort_action = 'SOA_EPM_abort_action'
    SOA_EPM_perform_action = 'SOA_EPM_perform_action'
    SOA_EPM_add_attachment_action = 'SOA_EPM_add_attachment_action'


class SoaEPMSupportingValues(Enum):
    """
    SoaEPMSupportingValues
    """
    SOA_EPM_no_decision = 'SOA_EPM_no_decision'
    SOA_EPM_approve = 'SOA_EPM_approve'
    SOA_EPM_reject = 'SOA_EPM_reject'
    SOA_EPM_unset = 'SOA_EPM_unset'
    SOA_EPM_completed = 'SOA_EPM_completed'
    SOA_EPM_unable_to_complete = 'SOA_EPM_unable_to_complete'
    SOA_EPM_true = 'SOA_EPM_true'
    SOA_EPM_false = 'SOA_EPM_false'
    SOA_EPM_no_error = 'SOA_EPM_no_error'


class stateType(Enum):
    """
    stateType
    """
    open_notrunning = 'open_notrunning'
    open_notrunning_suspended = 'open_notrunning_suspended'
    open_running = 'open_running'
    closed_completed = 'closed_completed'
    closed_abnormalcompleted = 'closed_abnormalcompleted'
    closed_abnormalcompleted_terminated = 'closed_abnormalcompleted_terminated'
    closed_abnormalcompleted_aborted = 'closed_abnormalcompleted_aborted'
    open_notrunning_pending = 'open_notrunning_pending'
    open_running_started = 'open_running_started'
    open_running_failed = 'open_running_failed'



from __future__ import annotations

from tcsoa.gen.Internal.Search._2020_12.SearchFolder import GetSearchFolderAccessorsOutput, ExportSearchFoldersResponse, ImportInput, CreateOrEditSearchFoldersOutput, ReportSearchRecipeInput, ReportSearchRecipeResponse, ImportSearchFolderResponse, SearchFoldersInput
from typing import List
from tcsoa.utils import TcService


class SearchFolderService(TcService):

    @classmethod
    def getSearchFolderAccessors(cls, searchFolders: List[str]) -> GetSearchFolderAccessorsOutput:
        """
        Returns list of accessors for each of the provided Awp0SearchFolder instance. An Awp0SearchFolder can be shared
        with different organizational entities like users, groups, roles. The list of accessors represent these
        entities with whom the folder has been shared.
        
        Use cases:
        When a user selects an Awp0SearchFolder instance, the list of accessors having share access on the folder
        should also be displayed. Client will make a call to this operation to get the accessors list.
        """
        return cls.execute_soa_method(
            method_name='getSearchFolderAccessors',
            library='Internal-Search',
            service_date='2020_12',
            service_name='SearchFolder',
            params={'searchFolders': searchFolders},
            response_cls=GetSearchFolderAccessorsOutput,
        )

    @classmethod
    def getTranslatedReportSearchRecipe(cls, reportDefinitionCriteria: List[ReportSearchRecipeInput]) -> ReportSearchRecipeResponse:
        """
        Generates target data source compatible search recipe and parameters for a given list of Teamcenter
        ReportDefinition objects. The returned search recipes can be directly consumed by external data sources, like
        Solr, without additional help from Teamcenter. A ReportDefinition object holds platform independent search
        recipe. When this object is opened or executed from within Teamcenter applications, the recipe is first
        translated to work with the target data source and then necessary search actions are performed. This operation
        provides translated search parameters to the caller so that search actions associated with Reports can be done
        outside of Teamcenter.
        
        Use cases:
        Currently Teamcenter product data can be indexed in Apache Solr search engine as part of Active Workspace
        solution to facilitate full text search. The Active Workspace global search feature allows users to find and
        load Teamcenter objects indexed in Solr. To take further advantage of the indexed Teamcenter data in Solr, a
        microservice to generate charting information for a requested Teamcenter property has been introduced. The
        microservice expects a Solr compatible query to be sent as part of the request, along with the property for
        which charting information is desired. It then executes the query and sends back analytical information which
        the client can display as charts. Operation getTranslatedReportSearchRecipe provides the calling client
        information necessary to make a request to the microservice. 1.    Client calls the
        getTranslatedReportSearchRecipe operation to obtain Solr compatible translated search parameters for a group of
        ReportDefinition objects. 
        2.    Once the client obtains the necessary information, it makes asynchronous call to the microservice using
        the translated search parameters.
        3.    Microservice performs the necessary search action, puts together charting response for the requested
        ReportDefinition objects and sends it back to the client.
        4.    Client can then render the obtained data as charts.
        Note: Since the whole search and chart generation process does not involve Teamcenter, live update of the
        Report objects can be done asynchronously and without blocking the client from executing any other actions.
        """
        return cls.execute_soa_method(
            method_name='getTranslatedReportSearchRecipe',
            library='Internal-Search',
            service_date='2020_12',
            service_name='SearchFolder',
            params={'reportDefinitionCriteria': reportDefinitionCriteria},
            response_cls=ReportSearchRecipeResponse,
        )

    @classmethod
    def importSearchFolder(cls, input: ImportInput) -> ImportSearchFolderResponse:
        """
        Imports a TCXML file containing Awp0SearchFolder object with its hierarchy and its associated ReportDefinition
        and Awp0SearchFolderShareRule objects.
        
        Use cases:
        Importing an Awp0SearchFolder hierarchy under an existing Awp0SearchFolder object
        
        1. In Active Workspace client, user selects an existing Awp0SearchFolder object and clicks on the Import Active
        Folder command. A panel is displayed where the user can specify input TCXML file to import. 
        
        2. Once the user chooses a valid file, a preview of the Awp0SearchFolder hierarchy that is to be imported is
        rendered. 
        
        3. After the preview is complete, a button to import the file is displayed. When the user clicks on the import
        button, the folder hierarchy will be imported under the selected Awp0SearchFolder object.
        
        Note: If an existing Awp0SearchFolder object is not selected during import, the hierarchy read from the input
        TCXML file will be created under the user&rsquo;s Awp0MySearchFolder object.
        """
        return cls.execute_soa_method(
            method_name='importSearchFolder',
            library='Internal-Search',
            service_date='2020_12',
            service_name='SearchFolder',
            params={'input': input},
            response_cls=ImportSearchFolderResponse,
        )

    @classmethod
    def createOrEditSearchFolders(cls, input: List[SearchFoldersInput]) -> CreateOrEditSearchFoldersOutput:
        """
        Creates or edits one or more Awp0SearchFolder objects. An Awp0SearchFolder object is a subtype of Folder with
        an integrated search definition. It displays dynamic contents based on the search definition it contains.
        Internally, Awp0SearchFolder' s search definition is represented by a ReportDefinition object. This operation
        supports creation and modification of Awp0SearchFolder's with necessary inputs.
        
        Use cases:
        Create Active Folders: User needs to create one or more instances of Awp0SearchFolder with their respective
        search definitions. Edit existing instances of Awp0SearchFolder: User needs to modify existing Active Folder
        instances. This can include updating the existing search definition or other Search Folder attributes like name
        or description.
        """
        return cls.execute_soa_method(
            method_name='createOrEditSearchFolders',
            library='Internal-Search',
            service_date='2020_12',
            service_name='SearchFolder',
            params={'input': input},
            response_cls=CreateOrEditSearchFoldersOutput,
        )

    @classmethod
    def exportSearchFolders(cls, searchFolderUIDs: List[str]) -> ExportSearchFoldersResponse:
        """
        Exports one or more Awp0SearchFolder objects and its hierarchy. Every valid instance of Awp0SearchFolder is
        exported with its associated ReportDefinition and Awp0SearchFolderShareRule.
        
        Use cases:
        1. In Active Workspace, users will click on an Awp0SearchFolder object and on clicking the "Export Active
        Folder" command, an exported TCXML will be send back to the client through a transient file ticket in the
        service response, which the client  uses to download the exported file.
        """
        return cls.execute_soa_method(
            method_name='exportSearchFolders',
            library='Internal-Search',
            service_date='2020_12',
            service_name='SearchFolder',
            params={'searchFolderUIDs': searchFolderUIDs},
            response_cls=ExportSearchFoldersResponse,
        )



from __future__ import annotations

from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass
from typing import Dict, List


@dataclass
class GetSessionInfoFromTicketResponse(TcBaseObj):
    """
    Structure that holds the user, session information and service data. The user and session information is captured
    in the call to the related getSessionInfoTicket and will be decoded from the encrypted sessionInfoTicket input
    string.
    
    :var userInfo: User login information, like: userId, userUid, groupName, groupUid, roleName, roleUid, projectId,
    projectUid
    :var sessionValues: sessionValues
    :var siteId: Site identifier that generated the ticket.
    :var serviceData: ServiceData  in which the partial errors are communicated to the client.
    """
    userInfo: UserInfo = None
    sessionValues: SessionValuesMap2 = None
    siteId: int = 0
    serviceData: ServiceData = None


@dataclass
class GetSessionInfoTicketResponse(TcBaseObj):
    """
    GetSessionInfoTicketResponse structure contains an encrypted ticket which has to be used to retrieve session info.
    
    :var sessionInfoTicket: An encrypted ticket in string format. Client can preserve this ticket and pass this ticket
    to getSessionInfoFromTicket to get the session info back.
    :var serviceData: ServiceData in which the partial errors are communicated to the client.
    """
    sessionInfoTicket: str = ''
    serviceData: ServiceData = None


"""
session values map2
"""
SessionValuesMap2 = Dict[str, List[str]]


"""
User login information, like: user_id, user_uid, group_id, group_uid, user_role_id, user_role_uid, user_project_id, user_project_uid.
"""
UserInfo = Dict[str, str]



from tcsoa.gen.Internal_Structure._2020_12.services import WhereUsedService as imp0
from tcsoa.utils import TcService


class WhereUsedService(TcService):
    getWhereUsedInfo = imp0.getWhereUsedInfo



from __future__ import annotations

from tcsoa.gen.Internal.GlobalMultiSite._2018_11.Briefcase import NamesAndValues, CheckBriefcaseLicenseResponse, GetBriefcaePreviewDataResponse
from typing import List
from tcsoa.utils import TcService


class BriefcaseService(TcService):

    @classmethod
    def checkBriefcaseLicense(cls) -> CheckBriefcaseLicenseResponse:
        """
        Operation checks if the current site has licenses, "multisite_server" and "tc_briefcase" for all briefcase
        add-on features such as "preview briefcase", "compare briefcases", "diverse schema".
        
        Use cases:
        Briefcase add on license is required for below user actions-
        - A user chooses a briefcase file in Teamcenter and wants to open it for Meta-data preview.
        - A user chooses two briefcase files in Teamcenter and wants to preview and compare them.
        - A user chooses to Import/Export data to managed briefcase in Teamcenter and transformer is used for this
        Import/Export.
        
        """
        return cls.execute_soa_method(
            method_name='checkBriefcaseLicense',
            library='Internal-GlobalMultiSite',
            service_date='2018_11',
            service_name='Briefcase',
            params={},
            response_cls=CheckBriefcaseLicenseResponse,
        )

    @classmethod
    def getBriefcasePreviewData(cls, oldBriefcaseFMSTicket: str, oldBriefcaseUID: str, newBriefcaseFMSTicket: str, newBriefcaseUID: str, optionNamesAndValues: List[NamesAndValues]) -> GetBriefcaePreviewDataResponse:
        """
        This operation is used to provide the structures in briefcases for preview and compare purpose. A briefcase may
        contain a structure in Teamcenter XML format. This operation parses the Teamcenter XML files the input
        briefcases, get the structure information and convert them to the data which can be used for preview and
        compare on the client side.
        
        Use cases:
        &bull;    A user chooses a briefcase file in Teamcenter and wants to open it for preview. The selected
        briefcase UID is sent to this operation. The operation parses it to get the data structure and returns it to
        the client side.
        &bull;    A user opens a briefcase file at the client side for preview. The briefcase file at the client side
        should be uploaded to the server before calling this operation. The FMS ticket of the uploaded file should be
        sent to this operation. The operation parses the briefcase to get the data structure and returns it to the
        client side. 
        &bull;    A user chooses two briefcase files in Teamcenter and wants to preview and compare them. The selected
        briefcase UIDs are sent to this operation. The operation parses the briefcases to get their data structure and
        returns them to the client side. The operation can also provide the delta data for the difference between these
        two briefcase files.
        &bull;    A user opens two briefcase files at client side and wants to preview and compare. The files at the
        client side should be uploaded to the server before calling this operation. The FMS tickets of the uploaded
        files should be sent to this operation. The operation parses the briefcase files to get their data structures
        and returns them to the client side. The operation can also provide the delta data for the difference between
        these two briefcase files.
        """
        return cls.execute_soa_method(
            method_name='getBriefcasePreviewData',
            library='Internal-GlobalMultiSite',
            service_date='2018_11',
            service_name='Briefcase',
            params={'oldBriefcaseFMSTicket': oldBriefcaseFMSTicket, 'oldBriefcaseUID': oldBriefcaseUID, 'newBriefcaseFMSTicket': newBriefcaseFMSTicket, 'newBriefcaseUID': newBriefcaseUID, 'optionNamesAndValues': optionNamesAndValues},
            response_cls=GetBriefcaePreviewDataResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, Dataset, ImanRelation
from typing import Dict, List
from tcsoa.gen.Internal.Visualization._2008_06.DataManagement import NamedRefsInDataset
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class Snapshot3DInfoInput2(TcBaseObj):
    """
    Input structure for getting SnapShotViewData Dataset info.
    
    :var clientId: Identifier for client data.
    :var snapshot3DDataset: Object of the input SnapShotViewData Dataset.
    :var getStaticFilesInfo: If true, it will return the static information of this Product View. This may not be
    available if a static representation of the Product View was not saved, or the data model does not support it.
    """
    clientId: str = ''
    snapshot3DDataset: Dataset = None
    getStaticFilesInfo: bool = False


@dataclass
class Snapshot3DInfoOutput2(TcBaseObj):
    """
    Output structure for the info contained in the Product View.
    
    :var namedRefFileInfoList: A list of file info named referenced by the SnapShotViewData Dataset.
    :var relObjList: A list of objects related to SnapShotViewData Dataset via GRM.
    :var snapshot3DVisibleLines: A list of visible lines in the Product View.
    """
    namedRefFileInfoList: List[NamedRefsInDataset] = ()
    relObjList: List[Snapshot3DRelObjInfo2] = ()
    snapshot3DVisibleLines: Snapshot3DVisibleLines2 = None


@dataclass
class Snapshot3DInfoResponse2(TcBaseObj):
    """
    Return structure for Snapshot3DInfo.
    
    :var snapshot3DInfoMap: A map (string, Snapshot3DInfoOutput2) containing client IDs and output structure as
    key/value pairs. The keys corresponding to the clientId specified in the Snapshot3DInfoInput2 and the values are a
    structure of Snapshot3DInfoOutput2 data.
    :var serviceData: Standard ServiceData member
    """
    snapshot3DInfoMap: Snapshot3DInfoOutputMap2 = None
    serviceData: ServiceData = None


@dataclass
class Snapshot3DRelObjInfo2(TcBaseObj):
    """
    List of objects related to SnapShotViewData Dataset via GRM.
    
    :var relType: Currently two values are supported:  TYPE_NamedRef and TYPE_GRM.
    :var relObj: Object Reference attached to the Dataset.
    :var relation: GRM relation on the attached Dataset such as VISTopLevelRef or IMAN_3D_snap_shot. For namedRef, it
    is Root Context representing the top line of the structure that must be loaded in order to display the Product View.
    """
    relType: str = ''
    relObj: BusinessObject = None
    relation: ImanRelation = None


@dataclass
class Snapshot3DVisibleLines2(TcBaseObj):
    """
    List of visible lines in the Product View.
    
    :var uidType: This supports two values: TYPE_CS (cloneStable unique id chain string) and TYPE_APN ( absolute path
    node unique id string).
    :var visibleLinesMap: A map (string, VisibleLine2) of visible BOMLine objects. The key is the parentRef from
    another VisibleLine struct.
    :var relDatasetInfoMap: A map (string, StaticFileInfo2) of the Dataset info related to the visible lines. The
    relDatasetRef string in the VisibleLine references the keys in this map.
    """
    uidType: str = ''
    visibleLinesMap: Snapshot3DVisibleLinesMap2 = None
    relDatasetInfoMap: Snapshot3DRelDatasetInfoMap2 = None


@dataclass
class StaticFileInfo2(TcBaseObj):
    """
    Structure to hold the file info from the static capture of the snapshot.
    
    :var datasetUID: Direct Model dataset UID for the JT part file on the visible line.
    :var imanFileUID: The ImanFile UID for the JT part file on the visible line. If there are multiple files in the
    Dataset, the first one is returned.
    :var originalFileName: Full file name plus extensions.
    """
    datasetUID: str = ''
    imanFileUID: str = ''
    originalFileName: str = ''


@dataclass
class VisibleLine2(TcBaseObj):
    """
    Structure to hold the unique data about the Visible BOMLine.
    
    :var uidString: The UID string of the Visible BOMLine objects, its type is specified by uidType.
    :var relDatasetRef: This is a string mapping to a related Dataset if not empty. Use the string as a key to the
    relDatasetInfo member in Snapshot3DVisibleLines structure.
    :var parentRef: This is the reference ID of the parent visible line (parentRef).
    :var altKeyType: This represents the type of the altKeyValue. Currently the only supported type is TYPE_IDIC.
    :var altKeyValue: Currently the only supported value is "ID in TopLevel context" (IDIC). This value will be
    populated for visible lines.
    """
    uidString: str = ''
    relDatasetRef: str = ''
    parentRef: str = ''
    altKeyType: str = ''
    altKeyValue: str = ''


"""
A map (string, Snapshot3DInfoOutput2) containing client IDs and output structure as key/value pairs. The keys corresponding to the clientId specified in the Snapshot3DInfoInput2 and the values are a structure of Snapshot3DInfoOutput2 data.
"""
Snapshot3DInfoOutputMap2 = Dict[str, Snapshot3DInfoOutput2]


"""
A map (string, StaticFileInfo2) of the Dataset info related to the visible lines. The relDatasetRef string in the VisibleLine references the keys in this map.
"""
Snapshot3DRelDatasetInfoMap2 = Dict[str, StaticFileInfo2]


"""
A map (string, VisibleLine2) of visible BOMLine objects. The key is the parentRef from another VisibleLine struct.
"""
Snapshot3DVisibleLinesMap2 = Dict[str, VisibleLine2]



from __future__ import annotations

from tcsoa.gen.Internal.MultiSite._2007_06.RemoteOperation import DateElement, IdsmIeFailures, SendObjectsSubstitutions, SendObjectsRetentions, IdsmObjectDesc, IdsmIeOpt
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class IDSM11CheckSyncStatusIn(TcBaseObj):
    """
    'IDSM11CheckSyncStatusIn' structure holds the information required to retrieve the sync status of the business
    object(s).
    
    :var clientInfo: Object having the client information.
    :var commandToken: Type of operation needs to be performed. The below enum lists the supported operations.
    
    'enum' IDSM_sync_status_command_e
    {
        IDSM_CHECK_EXISTS,
        IDSM_CHECK_SYNCHRONIZABLE,
        IDSM_CHECK_IN_SYNC,
        IDSM_CHECK_IN_SYNC_GMT
    }
    
    :var objectsToCheck: List of PUIDs of the objects in consideration.
    :var lmdDates: Last modified dates of the respective objects.
    """
    clientInfo: IDSM11ClientInfo = None
    commandToken: int = 0
    objectsToCheck: List[str] = ()
    lmdDates: List[DateElement] = ()


@dataclass
class IDSM11CheckSyncStatusOut(TcBaseObj):
    """
    'IDSM11CheckSyncStatusOut' structure holds the business object sync status code and the failure code occurred
    during the sync operation.
    
    :var statusCodes: It is status of object w.r.t to IDSM server 
    - 0 == does not exist
    - 1 == exists
    - 2 == exists as a readonly copy
    - 3 == exists as a stub.
    
    
    :var failureCodes: List of ifail failure codes, occurred during the sync operation.
    """
    statusCodes: List[int] = ()
    failureCodes: List[int] = ()


@dataclass
class IDSM11CheckSyncStatusRes(TcBaseObj):
    """
    'IDSM11CheckSyncStatusRes' structure holds the information of the business object sync status with respect to the
    IDSM server.
    
    :var out: Sync state of the object, along with the failure codes.
    :var serviceData: Any failure is returned in the ServiceData list of partial errors with input object mapped to
    error message.
    :var serverCorrelationId: Correlation id of the IDSM server.
    :var userDataIp: Currently this attribute not used, it is added for future use.
    """
    out: IDSM11CheckSyncStatusOut = None
    serviceData: ServiceData = None
    serverCorrelationId: str = ''
    userDataIp: str = ''


@dataclass
class IDSM11ClientInfo(TcBaseObj):
    """
    'IDSM11ClientInfo' structure holds the client information required to access the remote IDSM server.
    
    :var clientSiteId: Unique id allocated to client Teamcenter server site.
    :var targetSiteId: IDSM server site ID.
    :var clientNode: Machine name where Teamcenter Server is running.
    :var userId: Teamcenter user ID.
    :var userName: Teamcenter username.
    :var osUserName: Username allocated to user for accessing operating system.
    :var groupName: Teamcenter group associated with the given user name.
    :var roleName: Teamcenter role associated with the given user name.
    :var clientCorrelationId: Correlation id of requesting client.
    :var userDataIp: Currently this attribute not used, it is added for future use.
    """
    clientSiteId: int = 0
    targetSiteId: int = 0
    clientNode: str = ''
    userId: str = ''
    userName: str = ''
    osUserName: str = ''
    groupName: str = ''
    roleName: str = ''
    clientCorrelationId: str = ''
    userDataIp: str = ''


@dataclass
class IDSM11DescribeItemIn(TcBaseObj):
    """
    'IDSM11DescribeItemIn' structure holds the information required to retrieve the item s attribute value(s).
    
    :var clientInfo: An object having the requesting client information.
    :var itemId: Unique item id.
    """
    clientInfo: IDSM11ClientInfo = None
    itemId: str = ''


@dataclass
class IDSM11DescribeItemOut(TcBaseObj):
    """
    'IDSM11DescribeItemOut' structure holds the values and names of attributes retrieved from the IDSM server.
    
    :var attributes: List provides values for an item attribute name.
    :var attributeNames: List provides the available attribute names for an item.
    """
    attributes: List[str] = ()
    attributeNames: List[str] = ()


@dataclass
class IDSM11DescribeItemRes(TcBaseObj):
    """
    'IDSM11DescribeItemRes' structure provides the item information retrieved from IDSM server.
    
    :var out: An object with the item description.
    :var serviceData: Any failure is returned in the ServiceData list of partial errors with input object mapped to
    error message.
    :var serverCorrelationId: Correlation id of the IDSM server.
    :var userDataIp: Currently this attribute not used, currently it is added for future use.
    """
    out: IDSM11DescribeItemOut = None
    serviceData: ServiceData = None
    serverCorrelationId: str = ''
    userDataIp: str = ''


@dataclass
class IDSM11DescribeObjectIn(TcBaseObj):
    """
    'IDSM11DescribeObjectIn' structure holds the information required to retrieve the business object s attribute
    value(s).
    
    :var clientInfo: An object having the client information.
    :var tagsAsString: Object tag unique for the Teamcenter server session.
    """
    clientInfo: IDSM11ClientInfo = None
    tagsAsString: str = ''


@dataclass
class IDSM11DescribeObjectOut(TcBaseObj):
    """
    'IDSM11DescribeObjectOut' structure holds the description of object hold at IDSM server.
    
    :var objectDesc: The attribute provides the complete information of the object.
    """
    objectDesc: IdsmObjectDesc = None


@dataclass
class IDSM11DescribeObjectRes(TcBaseObj):
    """
    'IDSM11DescribeObjectRes' structure provides the object information retrieved from IDSM server.
    
    :var out: An object with the it s description, comprising  of site id, object tag, creataion date and last modified
    date as main constituent.
    :var serviceData: Any failure is returned in the ServiceData list of partial errors with input object mapped to
    error message.
    :var serverCorrelationId: Correlation id of the IDSM server.
    :var userDataIp: Currently this attribute not used, currently it is added for future use.
    """
    out: IDSM11DescribeObjectOut = None
    serviceData: ServiceData = None
    serverCorrelationId: str = ''
    userDataIp: str = ''


@dataclass
class IDSM11DistributedAppIn(TcBaseObj):
    """
    'IDSM11DistributedAppIn' structure holds the client information required to make the distributed call.
    
    :var clientInfo: An object having the client information.
    :var imanVersion: Teamcenter client version, internally this is converted to the client major version and minor
    version.
    :var imanIrmNumber: Teamcenter patch number.
    :var appName: Name of the application, client want to check service for.
    :var appOpCode: Op code of the service.
    DIST_IDSM_commit_synchronous_remote_import_op          0
    DIST_IDSM_exchange_supported_feature_set_op            1
    DIST_IDSM_ask_imported_sst_uids_op                     2
    DIST_IDSM_perform_checkpoint_function_op               3
    DIST_IDSM_identify_sst_client_op                       4
    DIST_IDSM_is_process_dead_op                           5
    DIST_IDSM_ask_server_process_info_op                   6
    DIST_IDSM_describe_object_l10n_attr_op                 7
    DIST_IDSM_start_plmxml_sync_op                         8
    DIST_IDSM_set_remote_site_logger_level_op           9
    
    :var appInputStrings: List of input strings for the controller.
    """
    clientInfo: IDSM11ClientInfo = None
    imanVersion: int = 0
    imanIrmNumber: int = 0
    appName: str = ''
    appOpCode: int = 0
    appInputStrings: List[str] = ()


@dataclass
class IDSM11DistributedAppOut(TcBaseObj):
    """
    'IDSM11DistributedAppOut' structure holds the strings returned by the controller.
    
    :var appOutputStrings: List of strings returned by the controller.
    """
    appOutputStrings: List[str] = ()


@dataclass
class IDSM11DistributedAppRes(TcBaseObj):
    """
    'IDSM11DistributedAppRes' structure holds the description of the service.
    
    :var out: An object with the service description.
    :var serviceData: Any failure is returned in the ServiceData list of partial errors with input object mapped to
    error message.
    :var serverCorrelationId: Correlation id of the IDSM server.
    :var userDataIp: Currently this attribute not used, currently it is added for future use.
    """
    out: IDSM11DistributedAppOut = None
    serviceData: ServiceData = None
    serverCorrelationId: str = ''
    userDataIp: str = ''


@dataclass
class IDSM11EndAskInfoIn(TcBaseObj):
    """
    'IDSM11EndAskInfoIn' structure holds the necessary information to finish the info file transfer.
    
    :var clientInfo: Correlation id of the IDSM server.
    :var infoType: This is not used internally.
    :var infoStagingDir: Complete path of the staging directory.
    :var abortFlag: Flag to determine the aborted operation.
    """
    clientInfo: IDSM11ClientInfo = None
    infoType: int = 0
    infoStagingDir: str = ''
    abortFlag: bool = False


@dataclass
class IDSM11EndExportItemIdListIn(TcBaseObj):
    """
    'IDSM11EndExportItemIdListIn' structure holds the necessary information to finish the remote item id import
    operation.
    
    :var clientInfo: An object having the requesting client information.
    :var exportStagingDir: Path of the export staging directory.
    """
    clientInfo: IDSM11ClientInfo = None
    exportStagingDir: str = ''


@dataclass
class IDSM11EndImportIn(TcBaseObj):
    """
    'IDSM11EndImportIn' structure holds information required to complete the remote export transaction.
    
    :var clientInfo: Object having the client information.
    :var importObjects: PUIDs of the objects to be imported.
    :var excludeObjects: PUIDs of the objects to be excluded.
    :var importOptions: Import options.
    :var importStagingDir: Complete path of the import staging directory.
    :var abortFlag: Flag to determine the aborted operation.
    :var compressPerformedFlg: Compression flag to determine if compression of the meta file requires.
    """
    clientInfo: IDSM11ClientInfo = None
    importObjects: List[str] = ()
    excludeObjects: List[str] = ()
    importOptions: List[IdsmIeOpt] = ()
    importStagingDir: str = ''
    abortFlag: bool = False
    compressPerformedFlg: int = 0


@dataclass
class IDSM11EndImportOut(TcBaseObj):
    """
    'IDSM11EndImportOut' structure holds the failures occurred during the remote export transaction.
    
    :var failures: List of failures on IDSM server, the list contain PUID of the failed object along with the ifail
    value.
    """
    failures: List[IdsmIeFailures] = ()


@dataclass
class IDSM11EndImportRes(TcBaseObj):
    """
    'IDSM11EndImportRes' structure holds the details required to access the remote IDSM server.
    
    :var out: List of IDSM failures.
    :var serviceData: Any failure is returned in the ServiceData list of partial errors with input object mapped to
    error message.
    :var serverCorrelationId: Correlation id of the IDSM server.
    :var userDataIp: Currently this attribute not used, it is added for future use.
    """
    out: IDSM11EndImportOut = None
    serviceData: ServiceData = None
    serverCorrelationId: str = ''
    userDataIp: str = ''


@dataclass
class IDSM11EndSendObjectsIn(TcBaseObj):
    """
    'IDSM11EndSendObjectsIn' structure holds the client information required to execute the distributed_execute utility
    remotely.
    
    :var clientInfo: Object having the client information.
    :var serverStagingDir: Complete path of the server staging directory.
    :var abortFlag: Flag to determine, if the export operation to be aborted after object send complete.
    """
    clientInfo: IDSM11ClientInfo = None
    serverStagingDir: str = ''
    abortFlag: bool = False


@dataclass
class IDSM11ExportStatusIn(TcBaseObj):
    """
    'IDSM11ExportStatusIn' structure holds information required to retrieve the status of ongoing import/export
    transaction.
    
    :var clientInfo: Object having the client information.
    :var exportStagingDir: Complete path of the staging directory.
    :var abortExport: Abort export operation.
    :var checkClientAfterNWsos: after no. of workspace objects, check for client, used for polling trigger.
    """
    clientInfo: IDSM11ClientInfo = None
    exportStagingDir: str = ''
    abortExport: bool = False
    checkClientAfterNWsos: int = 0


@dataclass
class IDSM11ExportStatusOut(TcBaseObj):
    """
    'IDSM11ExportStatusOut' structure holds the import/export status returned by 'idsm11ExportStatusSvc' operation. The
    structure also provides information on failures occurred during the remote import/export transaction.
    
    :var exportComplete: Boolean to check, export completion status.
    :var workspaceObjects: Number of workspace objects processed.
    :var exportFiles: List of export files created for workspace object transfer.
    :var failures: List of IDSM import export failures.
    :var exportIfail: Export completion ifail code. It is set to abort ifail code if client ask for the abort.
    :var compressPerformedFlg: The compression flag set to true in order to perform the compression during export.
    """
    exportComplete: bool = False
    workspaceObjects: int = 0
    exportFiles: List[str] = ()
    failures: List[IdsmIeFailures] = ()
    exportIfail: int = 0
    compressPerformedFlg: int = 0


@dataclass
class IDSM11ExportStatusRes(TcBaseObj):
    """
    'IDSM11ExportStatusRes' structure holds the import/export status returned by 'idsm11ExportStatusSvc' operation.
    
    :var out: Export state of the object.
    :var serviceData: Any failure is returned in the ServiceData list of partial errors with input object mapped to
    error message.
    :var serverCorrelationId: Correlation id of the IDSM server.
    :var userDataIp: Currently this attribute not used, it is added for future use.
    """
    out: IDSM11ExportStatusOut = None
    serviceData: ServiceData = None
    serverCorrelationId: str = ''
    userDataIp: str = ''


@dataclass
class IDSM11FailureRes(TcBaseObj):
    """
    'IDSM11FailureRes' structure provides the partial failures with error message.
    
    :var serviceData: Any failure is returned in the ServiceData list of partial errors with input object mapped to
    error message.
    :var serverCorrelationId: Correlation id of the IDSM server.
    :var userDataIp: Currently this attribute not used, currently it is added for future use.
    """
    serviceData: ServiceData = None
    serverCorrelationId: str = ''
    userDataIp: str = ''


@dataclass
class IDSM11GetErrorStackIn(TcBaseObj):
    """
    'IDSM11GetErrorStackIn' structure holds information required to retrieve the error stack.
    
    :var clientInfo: Object having the client information.
    :var extraInt: Extra integer to convey information to the server.
    :var extraString: Extra string to convery information to the server.
    """
    clientInfo: IDSM11ClientInfo = None
    extraInt: int = 0
    extraString: str = ''


@dataclass
class IDSM11GetErrorStackOut(TcBaseObj):
    """
    'IDSM11GetErrorStackOut' structure holds the error stack.
    
    :var nStackEntries: Number of errors in the error stack.
    :var errorCodes: List of error codes
    :var errorText: Error texts with respect to the error code.
    :var extraInt: Extra integer to convey the message to client. Now 1000 & 0 are used.
    :var extraString: Extra character(s) can be appended. E.g. The  \0 appended at the end.
    """
    nStackEntries: int = 0
    errorCodes: List[int] = ()
    errorText: List[str] = ()
    extraInt: int = 0
    extraString: str = ''


@dataclass
class IDSM11GetErrorStackRes(TcBaseObj):
    """
    'IDSM11GetErrorStackRes' structure holds the error stack along with the information required for logging.
    
    :var out: List of errors available in the stack.
    :var serviceData: Any failure is returned in the ServiceData list of partial errors with input object mapped to
    error message.
    :var serverCorrelationId: Correlation id of the IDSM server.
    :var userDataIp: Currently this attribute not used, it is added for future use.
    """
    out: IDSM11GetErrorStackOut = None
    serviceData: ServiceData = None
    serverCorrelationId: str = ''
    userDataIp: str = ''


@dataclass
class IDSM11LocateObjectIn(TcBaseObj):
    """
    'IDSM11LocateObjectIn' structure contains Integrated Distributed Services Manager (IDSM)               client info
    and input object tag which needs to be located.
    
    
    :var clientInfo: Object which has requesting client info.
    :var tagsAsString: Tag string of input object to be located.
    """
    clientInfo: IDSM11ClientInfo = None
    tagsAsString: str = ''


@dataclass
class IDSM11LocateObjectOut(TcBaseObj):
    """
    'IDSM11LocateObjectOut' structure holds owning site information and flag for object existence.
    
    :var found: Flag for object exists or not on remote site
    :var owningSiteId: Siteid of Owning site of an object.
    :var idsmSiteId: SiteId of IDSM server.
    """
    found: bool = False
    owningSiteId: int = 0
    idsmSiteId: int = 0


@dataclass
class IDSM11LocateObjectRes(TcBaseObj):
    """
    'IDSM11LocateObjectRes' structure holds locating information for input object and 'ServiceData'.
    
    :var out: Object which has owning site info of input object.
    :var serviceData: Any failure is returned in the ServiceData list of partial errors with input object mapped to
    error message.
    :var serverCorrelationId: Correlation id of the IDSM server.
    :var userDataIp: Currently this attribute not used, currently it is added for future use.
    """
    out: IDSM11LocateObjectOut = None
    serviceData: ServiceData = None
    serverCorrelationId: str = ''
    userDataIp: str = ''


@dataclass
class IDSM11PerformSendObjectsIn(TcBaseObj):
    """
    'IDSM11PerformSendObjectsIn' structure contains Integrated Distributed Services Manager 
    (IDSM) client info and other information required to send system objects to remote site.
    
    
    :var clientInfo: Object having which has requesting client info.
    :var serverStagingDir: Staging directory, files pushed from and pulled to this dir.
    :var emailAddresses: List of email addresses. Array of email addresses to which to send remote report.( Number of
    email addresses==0 implies no need to generate remote report file, unless required by 'report_file 'parameter or
    remote email preference.)
    
    :var substitutions: List of 'SendObjectsSubstitutions' objects which has attribute substitution specifications.
    
    :var retentions: List of 'SendObjectsRetentions' objects which has attribute retentions. 
    
    :var compressPerformedFlg: Compression flag to check if Compression is active.
    :var commandToPerform: Type of operation need to perform. The below 'enum' lists the supported operations.
    
    enum IDSM_send_sys_objects_cmd_e
    {
        IDSM_DSA_distribute_op,          //send to site to distribute                 
        IDSM_DSA_preview_distribute_op, // send to site to do a dry run distribute    
        IDSM_DSA_diff_op                // send to site to generate local diff report 
    } IDSM_send_sys_objects_cmd_t;
    
    :var commandOptionFlags: Operation command oprion flag.
    :var abortOnError: Flag to deteremine aborting on error.
    :var objectsXmlFile: XML objects file to send to remote site.
    :var plmxmlFileFormat: TRUE/FALSE, XML objects file is in plmxml format.
    :var objectsXmlFileOut: If sending objects for comparison or diff, name of XML file with remote objects to be
    pulled back.
    
    :var errorFile: Name of remote error file to pull back. (NULL implies no need to generate error file.).
    
    :var reportFile: Name of remote report file to pull back.  (NULL implies no need to  generate remote report file,
    unless required by email_addresses  or remote email preference.).
    """
    clientInfo: IDSM11ClientInfo = None
    serverStagingDir: str = ''
    emailAddresses: List[str] = ()
    substitutions: List[SendObjectsSubstitutions] = ()
    retentions: List[SendObjectsRetentions] = ()
    compressPerformedFlg: int = 0
    commandToPerform: int = 0
    commandOptionFlags: int = 0
    abortOnError: bool = False
    objectsXmlFile: str = ''
    plmxmlFileFormat: bool = False
    objectsXmlFileOut: str = ''
    errorFile: str = ''
    reportFile: str = ''


@dataclass
class IDSM11PerformSendObjectsOut(TcBaseObj):
    """
    'IDSM11PerformSendObjectsOut' structure contains export/import status information.
    
    :var returnFiles: List of output files to be returned to the client. It contains report file, status file etc. for
    export/import operation.
    
    :var compressSupportedFlg: Compression flag to check if Compression is active.
    """
    returnFiles: List[str] = ()
    compressSupportedFlg: int = 0


@dataclass
class IDSM11PerformSendObjectsRes(TcBaseObj):
    """
    'IDSM11PerformSendObjectsRes' structure provides information returned after sending objects to remote site and
    ServiceData.
    
    
    :var out: Object which has status info.
    :var serviceData: Any failure is returned in the ServiceData list of partial errors with input object mapped to
    error message.
    
    :var serverCorrelationId: Correlation id of the IDSM server.
    :var userDataIp: Currently this attribute not used, currently it is added for future use.
    """
    out: IDSM11PerformSendObjectsOut = None
    serviceData: ServiceData = None
    serverCorrelationId: str = ''
    userDataIp: str = ''


@dataclass
class IDSM11RemoteNotificationIn(TcBaseObj):
    """
    'IDSM11RemoteNotificationIn' structure contains Integrated Distributed Services Manager (IDSM)  client info and
    other  information required to notify remote site.
    
    
    :var clientInfo: Object which has requesting client info.
    :var eventTypeName: Event type string for sending notification type to the remote site.
    
    :var replicatedObjects: List of tag strings of input objects for which the notiifiction required to send at remote
    site.
    """
    clientInfo: IDSM11ClientInfo = None
    eventTypeName: str = ''
    replicatedObjects: List[str] = ()


@dataclass
class IDSM11RemoteNotificationOut(TcBaseObj):
    """
    'IDSM11StartExportItemIdListRes' structure provides failure code information.
    
    :var failureCodes: List of failure codes returned for each object given as an input.
    """
    failureCodes: List[int] = ()


@dataclass
class IDSM11RemoteNotificationRes(TcBaseObj):
    """
    'IDSM11RemoteNotificationRes' structure provides notification  information and 'ServiceData'.
    
    
    :var out: Object which has failure info.
    :var serviceData: Any failure is returned in the ServiceData list of partial errors with input object mapped to
    error message.
    
    :var serverCorrelationId: Correlation id of the IDSM server.
    :var userDataIp: Currently this attribute not used, currently it is added for future use.
    """
    out: IDSM11RemoteNotificationOut = None
    serviceData: ServiceData = None
    serverCorrelationId: str = ''
    userDataIp: str = ''


@dataclass
class IDSM11RepublishObjectIn(TcBaseObj):
    """
    IDSM11RepublishObjectIn structure contains Integrated Distributed Services Manager (IDSM) client info and other
    information required to re-publish object to ODS.
    
    
    
    :var clientInfo: Object having which has requesting client info.
    :var tagsAsString: Tag string of input object to be re-published.
    :var odsSiteId: Target ods siteid to which input object to be published.
    """
    clientInfo: IDSM11ClientInfo = None
    tagsAsString: str = ''
    odsSiteId: int = 0


@dataclass
class IDSM11SetSyncOptionsIn(TcBaseObj):
    """
    'IDSM11SetSyncOptionsIn' structure contains Integrated Distributed Services Manager(IDSM) client info and other
    information required to set sync options.
    
    
    :var clientInfo: Object which has requesting client info.
    :var optionsToken: Sync option token integer value. Below are the valid values of option token
    #define OBJIO_auto_sync                         1
    #define OBJIO_batch_sync                        2
    #define OBJIO_do_not_sync                       3
    #define OBJIO_auto_sync_and_notify            101
    #define OBJIO_batch_sync_but_notify           102
    #define OBJIO_do_not_sync_but_notify          103
    #define OBJIO_notify_and_sync_unchanged       104
    #define OBJIO_no_notify_and_sync_unchanged    105
    
    :var objectsOfInterest: List of object tag string for which sync options to be set.
    :var objectsNotifyOption: List of notification subscriptions option flag for each object in objectsOfInterest list.
    """
    clientInfo: IDSM11ClientInfo = None
    optionsToken: int = 0
    objectsOfInterest: List[str] = ()
    objectsNotifyOption: List[bool] = ()


@dataclass
class IDSM11SetSyncOptionsOut(TcBaseObj):
    """
    'IDSM11SetSyncOptionsOut' structure provides failure code info per object.
    
    :var failureCodes: List for failure code for each input object.
    """
    failureCodes: List[int] = ()


@dataclass
class IDSM11SetSyncOptionsRes(TcBaseObj):
    """
    'IDSM11SetSyncOptionsRes' structure provides information returned after setting sync options and ServiceData.
    
    :var out: Object which has failure info if any.
    :var serviceData: Any failure is returned in the ServiceData list of partial errors with input object mapped to
    error message.
    
    :var serverCorrelationId: Correlation id of the IDSM server.
    :var userDataIp: Currently this attribute not used, currently it is added for future use.
    """
    out: IDSM11SetSyncOptionsOut = None
    serviceData: ServiceData = None
    serverCorrelationId: str = ''
    userDataIp: str = ''


@dataclass
class IDSM11StartAskInfoIn(TcBaseObj):
    """
    'IDSM11StartAskInfoIn' structure contains Integrated Distributed Services Manager (IDSM) client info and other
    information required to start getting schema info.
    
    
    :var clientInfo: Object which has requesting client info.
    :var infoType: Info type flag as IDSM_schema_info to get type-class information which is required to be fetched
    from remote site.
    """
    clientInfo: IDSM11ClientInfo = None
    infoType: int = 0


@dataclass
class IDSM11StartAskInfoOut(TcBaseObj):
    """
    'IDSM11StartAskInfoOut' structure provides staging directory information.
    
    :var infoFiles: List which has name of the file which has to be pulled by client during file transfer.
    
    :var infoStagingDir: List which has name of the file which has to be pulled by client during file transfer.
    """
    infoFiles: List[str] = ()
    infoStagingDir: str = ''


@dataclass
class IDSM11StartAskInfoRes(TcBaseObj):
    """
    'IDSM11StartAskInfoRes' structure provides information required to start importing schema files and ServiceData.
    
    
    :var out: Object which has staging dierctory info.
    :var serviceData: Any failure is returned in the ServiceData list of partial errors with input object mapped to
    error message.
    
    :var serverCorrelationId: Correlation id of the IDSM server.
    :var userDataIp: Currently this attribute not used, currently it is added for future use.
    """
    out: IDSM11StartAskInfoOut = None
    serviceData: ServiceData = None
    serverCorrelationId: str = ''
    userDataIp: str = ''


@dataclass
class IDSM11StartExportIn(TcBaseObj):
    """
    'IDSM11StartExportIn' structure contains Integrated Distributed Services Manager (IDSM) client info and other
    information required for export.
    
    
    :var clientInfo: Object which has requesting client info.
    :var exportObjects: List of tag string of objects to be exported.
    :var excludeObjects: List of tag string of objects to be excluded during export.
    
    :var includeObjects: List of tag string of objects to be included during export.
    
    :var reason: User input to explain, why the remote import of business object required. Its not a mandatory input
    and user can pass empty string.
    
    :var exportOptions: List of 'IdsmIeOpt' structure object which has IDSM Import and Export options.
    
    :var compressFlg: Compression flag to check if Compression is active.
    :var compressType: Type of compression read  from IDSM_Compression_Type site level preference, its default value of
      InfoZip .
    
    :var checkClientAfterNWsos: After no. of workspace objects, check for client, used for polling trigger.
    """
    clientInfo: IDSM11ClientInfo = None
    exportObjects: List[str] = ()
    excludeObjects: List[str] = ()
    includeObjects: List[str] = ()
    reason: str = ''
    exportOptions: List[IdsmIeOpt] = ()
    compressFlg: int = 0
    compressType: str = ''
    checkClientAfterNWsos: int = 0


@dataclass
class IDSM11StartExportItemIdListIn(TcBaseObj):
    """
    'IDSM11StartExportItemIdListIn' structure contains Integrated Distributed Services Manager (IDSM) client info and
    other information required to start export itemids list.
    
    
    :var clientInfo: Object which has requesting client info.
    :var queryStrings: List of query strings for querying itemids.
    :var compressFlg: Compression flag to check if Compression is active.
    :var compressType: Type of compression read  from IDSM_Compression_Type site level preference, its default value of
      InfoZip .   
    """
    clientInfo: IDSM11ClientInfo = None
    queryStrings: List[str] = ()
    compressFlg: int = 0
    compressType: str = ''


@dataclass
class IDSM11StartExportItemIdListOut(TcBaseObj):
    """
    Return result of calling IDSM11StartExportItemIdList.
    
    :var exportFiles: exportFiles
    :var exportStagingDir: exportStagingDir
    :var compressPerformedFlg: compressPerformedFlg
    """
    exportFiles: List[str] = ()
    exportStagingDir: str = ''
    compressPerformedFlg: int = 0


@dataclass
class IDSM11StartExportItemIdListRes(TcBaseObj):
    """
    Return result of calling IDSM11StartExportItemIdList.
    
    :var out: out
    :var serviceData: serviceData
    :var serverCorrelationId: serverCorrelationId
    :var userDataIp: userDataIp
    """
    out: IDSM11StartExportItemIdListOut = None
    serviceData: ServiceData = None
    serverCorrelationId: str = ''
    userDataIp: str = ''


@dataclass
class IDSM11StartExportOut(TcBaseObj):
    """
    'IDSM11StartExportOut' structure contains information about Multi-site staging directory and compression flag.
    
    
    :var exportFiles: List of export files which has exported objects info created in export staging directory.
    
    :var exportStagingDir: Export staging directory path in which exportFiles files are pushed to and read from this
    directory.
    
    :var failures: List of 'IdsmIeFailures' structure object which has IDSM failure information.
    
    :var compressPerformedFlg: Compression flag to check whether exportFiles are compressed or not. IF compression is
    done set the correct flag for decompressing later in the flow.
    """
    exportFiles: List[str] = ()
    exportStagingDir: str = ''
    failures: List[IdsmIeFailures] = ()
    compressPerformedFlg: int = 0


@dataclass
class IDSM11StartExportRes(TcBaseObj):
    """
    'IDSM11StartExportRes' structure provides staging directory information for starting import  and 'ServiceData'.
    
    
    :var out: Object which has staging directory info.
    :var serviceData: Any failure is returned in the ServiceData list of partial errors with input object mapped to
    error message.
    
    :var serverCorrelationId: Correlation id of the IDSM server.
    :var userDataIp: Currently this attribute not used, currently it is added for future use.
    """
    out: IDSM11StartExportOut = None
    serviceData: ServiceData = None
    serverCorrelationId: str = ''
    userDataIp: str = ''


@dataclass
class IDSM11StartImportIn(TcBaseObj):
    """
    'IDSM11StartImportIn' structure contains Integrated Distributed Services Manager (IDSM) client info and required
    POM transmit file  Details, compression flag to start export.
    
    
    :var clientInfo: Object having which has requesting client info.
    :var xmitFname: Path of the multisite transmit file of exporting site copied to file store.
    
    :var compressFlg: Compression flag to check if Compression is active for input files.
    
    :var compressType: Type of compression read  from IDSM_Compression_Type site level preference, its default value of
      InfoZip .
    """
    clientInfo: IDSM11ClientInfo = None
    xmitFname: str = ''
    compressFlg: int = 0
    compressType: str = ''


@dataclass
class IDSM11StartImportOut(TcBaseObj):
    """
    'IDSM11StartImportOut' structure contains information about Multi-site staging directory and             
    compression flag.
    
    
    :var importStagingDir: Server/importing side staging directory path in which exported objects files are  being
    transferred on the server.
    
    :var sendXmitFile: TRUE/FALSE, flag for whether to send POM transmit file from server.
    
    :var compressSupportedFlg: Compression flag to check if Compression is active on server/importing  site or not.
    """
    importStagingDir: str = ''
    sendXmitFile: bool = False
    compressSupportedFlg: int = 0


@dataclass
class IDSM11StartImportRes(TcBaseObj):
    """
    'IDSM11StartImportRes' structure provides staging directory information for starting export and  Servicedata.
    
    
    :var out: Object which has staging directory info.
    :var serviceData: Any failure is returned in the ServiceData list of partial errors with input object mapped to
    error message.
    
    :var serverCorrelationId: Correlation id of the IDSM server.
    :var userDataIp: Currently this attribute not used, currently it is added for future use.
    """
    out: IDSM11StartImportOut = None
    serviceData: ServiceData = None
    serverCorrelationId: str = ''
    userDataIp: str = ''


@dataclass
class IDSM11StartSendObjectsIn(TcBaseObj):
    """
    'IDSM11StartSendObjectsIn' Integrated Distributed Services Manager (IDSM) client information and compression
    details.
    
    
    :var clientInfo: Object which has requesting client info.
    :var compressFlg: Compression flag to check if Compression is active or not.
    :var compressType: Type of compression read  from IDSM_Compression_Type site level preference, its default value of
      InfoZip .
    """
    clientInfo: IDSM11ClientInfo = None
    compressFlg: int = 0
    compressType: str = ''


@dataclass
class IDSM11StartSendObjectsOut(TcBaseObj):
    """
    'IDSM11StartSendObjectsOut' structure contains server side staging directory and compression flag.
    
    :var serverStagingDir: Server/importing side staging directory path in which object XML files are  being
    transferred on the server.
    
    :var compressSupportedFlg: Compression flag to check if Compression is active on server/importing  site or not.
    """
    serverStagingDir: str = ''
    compressSupportedFlg: int = 0


@dataclass
class IDSM11StartSendObjectsRes(TcBaseObj):
    """
    'IDSM11StartSendObjectsRes' structure contains information required to start the export and ServiceData.
    
    :var out: Object which has staging directory info.
    :var serviceData: Any failure is returned in the ServiceData list of partial errors with input object mapped to
    error message.
    
    :var serverCorrelationId: Correlation id of the IDSM server.
    :var userDataIp: Currently this attribute not used, it is added for future use.
    """
    out: IDSM11StartSendObjectsOut = None
    serviceData: ServiceData = None
    serverCorrelationId: str = ''
    userDataIp: str = ''


@dataclass
class IDSM11VerifyObjectsIn(TcBaseObj):
    """
    'IDSM11VerifyObjectsIn' structure holds information required to verify input object and Integrated Distributed
    Services Manager (IDSM) client info.
    
    :var clientInfo: Object which has requesting client info.
    :var objectsToVerify: List of object tag string  to be verified in the database.
    """
    clientInfo: IDSM11ClientInfo = None
    objectsToVerify: List[str] = ()


@dataclass
class IDSM11VerifyObjectsOut(TcBaseObj):
    """
    'IDSM11VerifyObjectsOut' structure contains object verification status and failure code if any.
    
    :var verdicts:  List of verdicts after verification of input objects. Below are the valid values of verdict        
       
    -                  0 = does not exist
    -                  1 = exists
    -                  2 = exists as a readonly copy
    -                  3 = exists as a stub
    
    
    :var failureCodes: List of failure codes if any for each input object.
    """
    verdicts: List[int] = ()
    failureCodes: List[int] = ()


@dataclass
class IDSM11VerifyObjectsRes(TcBaseObj):
    """
    'IDSM11VerifyObjectsRes' structure contains object verification information retrieved from IDSM server and
    'ServiceData'.
    
    :var out: Object contains input objects verification status.
    :var serviceData: Any failure is returned in the ServiceData list of partial errors with input object mapped to
    error message.
    
    :var serverCorrelationId: Correlation id of the IDSM server.
    :var userDataIp: Currently this attribute not used, currently it is added for future use.
    """
    out: IDSM11VerifyObjectsOut = None
    serviceData: ServiceData = None
    serverCorrelationId: str = ''
    userDataIp: str = ''


@dataclass
class IDSM11VersionCheckIn(TcBaseObj):
    """
    - 'IDSM11VersionCheckIn' structure holds information required to get the IDSM server version.
    
    
    
    :var clientInfo: Object which has requesting client info.
    :var clientVersion: Requesting client version.
    """
    clientInfo: IDSM11ClientInfo = None
    clientVersion: int = 0


@dataclass
class IDSM11VersionCheckOut(TcBaseObj):
    """
    'IDSM11VersionCheckOut' structure holds the IDSM server version information.
    
    :var serverVersion: IDSM server version.
    :var clientRejected: Client rejected status if IDSM rejects client for some reason.
    """
    serverVersion: int = 0
    clientRejected: int = 0


@dataclass
class IDSM11VersionCheckRes(TcBaseObj):
    """
    'IDSM11VersionCheckRes' structure holds the IDSM version information and ServiceData.
    
    :var out: Object contains version info.
    :var serviceData: Any failure is returned in the ServiceData list of partial errors with input object mapped to
    error message.
    
    :var serverCorrelationId: Correlation id of the IDSM server.
    :var userDataIp: Currently this attribute not used, currently it is added for future use.
    """
    out: IDSM11VersionCheckOut = None
    serviceData: ServiceData = None
    serverCorrelationId: str = ''
    userDataIp: str = ''


@dataclass
class IDSM11XferInfoFileIn(TcBaseObj):
    """
    'IDSM11XferInfoFileIn' structure holds the information required to transfer files to remote site and Integrated
    Distributed Services Manager (IDSM) client info.
    
    :var clientInfo: Object which has requesting client info.
    :var infoType: Info type flag as IDSM_schema_info to get type-class information which is required to be fetched
    from remote site.
    
    :var infoStagingDir: Server side staging directory path received in first call.
    :var infoFile: Info file which needs to be pulled in suring transfer.
    :var startingBlock: Transfer happens block by block, so index for starting block.
    """
    clientInfo: IDSM11ClientInfo = None
    infoType: int = 0
    infoStagingDir: str = ''
    infoFile: str = ''
    startingBlock: int = 0


@dataclass
class IDSM11XferInfoFileOut(TcBaseObj):
    """
    'IDSM11XferInfoFileOut' structure holds the file data information which is transferred to remote site.
    
    :var fileData: Actual file data which in the form of bytes.
    :var fileDataSize: File data size which is pulled from server.
    :var transferComplete: Flag to indicate whether all file transfer is complete or not.
    :var moreFiles: It transfers one file at a time, so flag to check whether there are more files to be pulled in.
    """
    fileData: List[int] = ()
    fileDataSize: int = 0
    transferComplete: bool = False
    moreFiles: bool = False


@dataclass
class IDSM11XferInfoFileRes(TcBaseObj):
    """
    'IDSM11XferInfoFileRes' structure holds the file transfer output information.
    
    :var out: Object contains file transfer info.
    :var serviceData: Any failure is returned in the ServiceData list of partial errors with input object mapped to
    error message.
    
    :var serverCorrelationId: Correlation id of the IDSM server.
    :var userDataIp: Currently this attribute not used, currently it is added for future use.
    """
    out: IDSM11XferInfoFileOut = None
    serviceData: ServiceData = None
    serverCorrelationId: str = ''
    userDataIp: str = ''



from tcsoa.gen.Internal_Requirementsmanagement._2012_10.services import RequirementsManagementService as imp0
from tcsoa.gen.Internal_Requirementsmanagement._2012_02.services import RequirementsManagementService as imp1
from tcsoa.gen.Internal_Requirementsmanagement._2009_10.services import RequirementsManagementService as imp2
from tcsoa.utils import TcService


class RequirementsManagementService(TcService):
    getMatchingLines = imp0.getMatchingLines
    getTraceabilityMatrix = imp1.getTraceabilityMatrix
    getTraceabilityMatrix2 = imp0.getTraceabilityMatrix
    importUnmanagedData = imp2.importUnmanagedData



from __future__ import annotations

from tcsoa.gen.Manufacturing._2009_10.ModelDefinitions import PropDescInfo, GetValidRelationTypesResponse, AttachedPropDescsResponse, RelationTypesInput
from tcsoa.gen.BusinessObjects import BusinessObject, StructureContext, WorkspaceObject
from tcsoa.gen.Manufacturing._2009_10.Model import ResolveData, GetResolvedNodesFromLAResponse, LogicalAssignmentData, CalculateCriticalPathResponse, FlowInput, FlowResponse, ResolvedNodesInput
from tcsoa.gen.Manufacturing._2009_10.MFGPropertyCollector import CollectPropertiesInputInfo, CollectPropertiesResponse
from tcsoa.gen.Manufacturing._2009_10.StructureManagement import GetStructureContextLinesResponse, PasteDuplicateStructureResponse, CopyEBOPStructureResponse
from tcsoa.gen.Manufacturing._2009_10.DataManagement import CreateIn, CreateResponse, DisconnectInput
from typing import List
from tcsoa.gen.Manufacturing._2009_10.StructureSearch import MFGSearchCriteria, StructureSearchResultResponse, SearchExpressionSet
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcService


class ModelService(TcService):

    @classmethod
    def getResolvedNodesFromLA(cls, inputObjects: List[ResolvedNodesInput]) -> GetResolvedNodesFromLAResponse:
        """
        This service returns the resolved nodes for each of the received Logical Assignment objects.
        """
        return cls.execute_soa_method(
            method_name='getResolvedNodesFromLA',
            library='Manufacturing',
            service_date='2009_10',
            service_name='Model',
            params={'inputObjects': inputObjects},
            response_cls=GetResolvedNodesFromLAResponse,
        )

    @classmethod
    def removeFlow(cls, input: List[BusinessObject]) -> ServiceData:
        """
        Removing existing Flow objects.
        """
        return cls.execute_soa_method(
            method_name='removeFlow',
            library='Manufacturing',
            service_date='2009_10',
            service_name='Model',
            params={'input': input},
            response_cls=ServiceData,
        )

    @classmethod
    def resolveLogicalAssignments(cls, laVec: List[BusinessObject], resolveObjects: List[ResolveData], externalSources: List[BusinessObject]) -> ServiceData:
        """
        This service will resolve and re-resolve logical assignments to concrete assignments against the product
        structure.
        """
        return cls.execute_soa_method(
            method_name='resolveLogicalAssignments',
            library='Manufacturing',
            service_date='2009_10',
            service_name='Model',
            params={'laVec': laVec, 'resolveObjects': resolveObjects, 'externalSources': externalSources},
            response_cls=ServiceData,
        )

    @classmethod
    def createFlow(cls, input: List[FlowInput]) -> FlowResponse:
        """
        Create a new mfgFlow object between two mfg objects i.e process or operation
        """
        return cls.execute_soa_method(
            method_name='createFlow',
            library='Manufacturing',
            service_date='2009_10',
            service_name='Model',
            params={'input': input},
            response_cls=FlowResponse,
        )

    @classmethod
    def editLogicalAssignments(cls, laEditVec: List[LogicalAssignmentData]) -> ServiceData:
        """
        This service enables editing the values of Logical Assignment objects.
        """
        return cls.execute_soa_method(
            method_name='editLogicalAssignments',
            library='Manufacturing',
            service_date='2009_10',
            service_name='Model',
            params={'laEditVec': laEditVec},
            response_cls=ServiceData,
        )

    @classmethod
    def calculateCriticalPath(cls, roots: List[BusinessObject]) -> CalculateCriticalPathResponse:
        """
        Calculate the critical paths for MFGBVRProcess, MFGBVROperation or MFGBVRActivity and their corresponding APS
        objects.
        A critical path is the sequence of processes, operations or activities that determine the minimum duration of
        the root object. Thereby only those MFG objects will be considered that are either direct sub elements or
        implementers of the root object.
        """
        return cls.execute_soa_method(
            method_name='calculateCriticalPath',
            library='Manufacturing',
            service_date='2009_10',
            service_name='Model',
            params={'roots': roots},
            response_cls=CalculateCriticalPathResponse,
        )


class StructureManagementService(TcService):

    @classmethod
    def getStructureContextLines(cls, scList: List[StructureContext]) -> GetStructureContextLinesResponse:
        """
        Return the top lines and any selected lines if present.
        """
        return cls.execute_soa_method(
            method_name='getStructureContextLines',
            library='Manufacturing',
            service_date='2009_10',
            service_name='StructureManagement',
            params={'scList': scList},
            response_cls=GetStructureContextLinesResponse,
        )

    @classmethod
    def pasteDuplicateStructure(cls, srcLines: List[BusinessObject], targetLines: List[BusinessObject], copyRulesKey: str, copyFutureEffectivity: bool) -> PasteDuplicateStructureResponse:
        """
        clone the selected lines to each of the targetline supplied. The 2 input vectors are not related. Currently,
        this functionality is only for EBOP structures.
        """
        return cls.execute_soa_method(
            method_name='pasteDuplicateStructure',
            library='Manufacturing',
            service_date='2009_10',
            service_name='StructureManagement',
            params={'srcLines': srcLines, 'targetLines': targetLines, 'copyRulesKey': copyRulesKey, 'copyFutureEffectivity': copyFutureEffectivity},
            response_cls=PasteDuplicateStructureResponse,
        )

    @classmethod
    def copyEBOPStructure(cls, newRoot: WorkspaceObject, configuringEBOPWindow: BusinessObject, workingWindow: BusinessObject, copyRulesKey: str, copyFutureEffectivity: bool) -> CopyEBOPStructureResponse:
        """
         Creates a clone of the supplied root of the configuringEBOPWindow under the rootObject specified.
        """
        return cls.execute_soa_method(
            method_name='copyEBOPStructure',
            library='Manufacturing',
            service_date='2009_10',
            service_name='StructureManagement',
            params={'newRoot': newRoot, 'configuringEBOPWindow': configuringEBOPWindow, 'workingWindow': workingWindow, 'copyRulesKey': copyRulesKey, 'copyFutureEffectivity': copyFutureEffectivity},
            response_cls=CopyEBOPStructureResponse,
        )


class ModelDefinitionsService(TcService):

    @classmethod
    def getValidRelationTypes(cls, relationTypesInput: List[RelationTypesInput]) -> GetValidRelationTypesResponse:
        """
        This service returns a list of occurrence types that are valid for assignment between two received object types.
        """
        return cls.execute_soa_method(
            method_name='getValidRelationTypes',
            library='Manufacturing',
            service_date='2009_10',
            service_name='ModelDefinitions',
            params={'relationTypesInput': relationTypesInput},
            response_cls=GetValidRelationTypesResponse,
        )

    @classmethod
    def getManufacturingPropretyDescs(cls, inputs: List[PropDescInfo]) -> AttachedPropDescsResponse:
        """
        Get the attached property descriptor based on input type name and property names structure.
        """
        return cls.execute_soa_method(
            method_name='getManufacturingPropretyDescs',
            library='Manufacturing',
            service_date='2009_10',
            service_name='ModelDefinitions',
            params={'inputs': inputs},
            response_cls=AttachedPropDescsResponse,
        )


class MFGPropertyCollectorService(TcService):

    @classmethod
    def collectProperties(cls, input: List[CollectPropertiesInputInfo]) -> CollectPropertiesResponse:
        """
        This function will call a Mfg function that takes the MfgNode, traversal rules and property names to collect
        and return a list of property values of input properties for every MfgNode in the BOP structure based on
        traversal rules.
        """
        return cls.execute_soa_method(
            method_name='collectProperties',
            library='Manufacturing',
            service_date='2009_10',
            service_name='MFGPropertyCollector',
            params={'input': input},
            response_cls=CollectPropertiesResponse,
        )


class StructureSearchService(TcService):

    @classmethod
    def nextSearch(cls, searchCursor: BusinessObject) -> StructureSearchResultResponse:
        """
        Process one additional step of the search identified by the cursor.
        """
        return cls.execute_soa_method(
            method_name='nextSearch',
            library='Manufacturing',
            service_date='2009_10',
            service_name='StructureSearch',
            params={'searchCursor': searchCursor},
            response_cls=StructureSearchResultResponse,
        )

    @classmethod
    def startSearch(cls, scope: List[BusinessObject], searchExpression: SearchExpressionSet, mfgSearchCriteria: MFGSearchCriteria) -> StructureSearchResultResponse:
        """
        Start searching a structure for a given search expression within the scope specified.
        search can also be narrowed to a specific object type, item name, and logical designator
        """
        return cls.execute_soa_method(
            method_name='startSearch',
            library='Manufacturing',
            service_date='2009_10',
            service_name='StructureSearch',
            params={'scope': scope, 'searchExpression': searchExpression, 'mfgSearchCriteria': mfgSearchCriteria},
            response_cls=StructureSearchResultResponse,
        )

    @classmethod
    def stopSearch(cls, searchCursor: BusinessObject) -> StructureSearchResultResponse:
        """
        Stop and close down a search identified by a cursor.
        Throws SearchAlreadyStoppedException if the search has already been stopped.
        """
        return cls.execute_soa_method(
            method_name='stopSearch',
            library='Manufacturing',
            service_date='2009_10',
            service_name='StructureSearch',
            params={'searchCursor': searchCursor},
            response_cls=StructureSearchResultResponse,
        )


class DataManagementService(TcService):

    @classmethod
    def createObjects(cls, input: List[CreateIn]) -> CreateResponse:
        """
        Generic operation for creation of manufacturing objects. This will also create any secondary(compounded)
        objects that need to be created, assuming the CreateInput for the secondary object is represented in the
        recursive CreateInput object e.g. Item is Primary Object that also creates ItemMasterForm, ItemRevision and
        ItemRevision in turn creates ItemRevisionMasterForm. The input for all these levels is passed in through the
        recursive CreateInput object.
        This operation creates the persistent objects and the runtime objects accordingly. This operation also connects
        the created objects to the specified target. The connection will be done by the relation defined as default.
        """
        return cls.execute_soa_method(
            method_name='createObjects',
            library='Manufacturing',
            service_date='2009_10',
            service_name='DataManagement',
            params={'input': input},
            response_cls=CreateResponse,
        )

    @classmethod
    def disconnectObjects(cls, input: List[DisconnectInput]) -> ServiceData:
        """
        Generic operation to disconnect objects.
        """
        return cls.execute_soa_method(
            method_name='disconnectObjects',
            library='Manufacturing',
            service_date='2009_10',
            service_name='DataManagement',
            params={'input': input},
            response_cls=ServiceData,
        )



from __future__ import annotations

from tcsoa.gen.Manufacturing._2018_06.DataManagement import GetOccKinematicsInfoInput, GetOccurrenceKinematicsInfoResponse, OccKinematicsInfoMap
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcService


class DataManagementService(TcService):

    @classmethod
    def setOccurrenceKinematicsInformation(cls, occInfoInputMap: OccKinematicsInfoMap) -> ServiceData:
        """
        In Line Designer (LD) and Process Simulate (PS), resource occurrence used has specific poses and joint values.
        This operation creates a relation between AbsOccData and Mfg0OccKinematicsInfo using relation
        Mfg0OccKinematicsRel. The occurrence kinematics information is stored as an XML reference on the dataset
        Mfg0OccKinematicsInfo
        
        Use cases:
        Line Designer or Process Simulate user wants to set the occurrence kinematics information for occurrence of
        Mfg0MEResourceRevision or ItemRevision
        """
        return cls.execute_soa_method(
            method_name='setOccurrenceKinematicsInformation',
            library='Manufacturing',
            service_date='2018_06',
            service_name='DataManagement',
            params={'occInfoInputMap': occInfoInputMap},
            response_cls=ServiceData,
        )

    @classmethod
    def getOccurrenceKinematicsInformation(cls, occKinematicsInfoinput: List[GetOccKinematicsInfoInput]) -> GetOccurrenceKinematicsInfoResponse:
        """
        In Line Designer (LD) and Process Simulate (PS), resource occurrence has specific poses and joint values. This
        operation retrieves occurrence kinematics information of Mfg0MEResourceRevision or ItemRevision from Bill of
        Equipment structure for the given scope Mfg0BvrWorkarea and context Mfg0BvrWorkarea acting as a root of the
        structure
        
        Use cases:
        Line Designer or Process Simulate user wants to get the occurrence kinematics information for occurrence of
        Mfg0MEResourceRevision or ItemRevision from Bill of Equipment structure
        """
        return cls.execute_soa_method(
            method_name='getOccurrenceKinematicsInformation',
            library='Manufacturing',
            service_date='2018_06',
            service_name='DataManagement',
            params={'occKinematicsInfoinput': occKinematicsInfoinput},
            response_cls=GetOccurrenceKinematicsInfoResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import WorkspaceObject, Awb0Element, ImanRelation
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class AttachObjectsRespData(TcBaseObj):
    """
    The relation created between the primary represented by Awb0Element and the secondary object.
    
    :var clientId: The unmodified value from the AttachObjectsInputData.clientId. This is used by the caller to
    identify this data structure with the source input data.
    :var relationObject: The newly created IMANRelation.
    """
    clientId: str = ''
    relationObject: ImanRelation = None


@dataclass
class AttachObjectsInputData(TcBaseObj):
    """
    AttachObjectsInputData Structure represents all required parameters to attach secondary object with primary object
    using given relation.
    
    :var clientId: This unique ID is used to identify return data elements and partial errors associated with this
    input structure.
    :var relationType: Name of the valid IMANRelation to create between primary and secondary.
    :var primary: The primary object for the relation.
    :var primaryContext: The context in which relation to be created. If NULL then relation is created with underlying
    business object as primary.
    :var secondary: The secondary object for the relation.
    """
    clientId: str = ''
    relationType: str = ''
    primary: Awb0Element = None
    primaryContext: Awb0Element = None
    secondary: WorkspaceObject = None


@dataclass
class AttachObjectsResp(TcBaseObj):
    """
    AttachObjectsResp structure represents the relations created between the primary and secondary object and errors
    occurred.
    
    :var output: A list of created IMANRelation.
    :var serviceData: Standard ServiceData object to hold the partial errors that the operation encounters.
    """
    output: List[AttachObjectsRespData] = ()
    serviceData: ServiceData = None



from __future__ import annotations

from tcsoa.gen.Requirementsmanagement._2012_09.RequirementsManagement import GetBOMLineInfo
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcService


class RequirementsManagementService(TcService):

    @classmethod
    def getBomlineAfterCreate(cls, inputs: List[GetBOMLineInfo]) -> ServiceData:
        """
        This operation creates a BOMLine for a newly created  Item and adds it to the selected parent BOMLine and
        checks out the latest revision of newly created Item based on a check-out preference. The inputs structure for
        this operation contains selected parent BOMLine and newly created Item (e.g. Functionality or Logical Block).
        """
        return cls.execute_soa_method(
            method_name='getBomlineAfterCreate',
            library='Requirementsmanagement',
            service_date='2012_09',
            service_name='RequirementsManagement',
            params={'inputs': inputs},
            response_cls=ServiceData,
        )



from tcsoa.gen.Internal_Classification._2020_04.services import ClassificationService as imp0
from tcsoa.gen.Internal_Classification._2017_05.services import ClassificationService as imp1
from tcsoa.gen.Internal_Classification._2009_10.services import ClassificationService as imp2
from tcsoa.gen.Internal_Classification._2018_11.services import ClassificationService as imp3
from tcsoa.utils import TcService


class ClassificationService(TcService):
    findClassificationInfo = imp0.findClassificationInfo
    getClassDefinitionsNX = imp1.getClassDefinitionsNX
    getClassificationHierarchies = imp2.getClassificationHierarchies
    getClassificationProperties = imp2.getClassificationProperties
    saveClassificationObjects = imp3.saveClassificationObjects



from __future__ import annotations

from tcsoa.gen.BusinessObjects import ImanFile
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class RenameIMFInfo(TcBaseObj):
    """
    Input structure that contains a clientID, file object reference, and string for rename.
    
    :var clientId: Input string to uniquely identify the input, used primarily for error handling and output mapping.
    :var file: File object to rename.
    :var name: Input string used to rename the input file.
    """
    clientId: str = ''
    file: ImanFile = None
    name: str = ''



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import Dict, List
from tcsoa.gen.Manufacturing._2009_10.StructureSearch import MFGSearchRefinementSet
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class GetAssignmentRecipesResp(TcBaseObj):
    """
    GetAssignmentRecipesResp structure contains a list of AdditionalInfo elements, the size of which matches the input
    recipeAnchor list. It also includes the standard serviceData object.
    
    :var info: A list of AdditionInfo structures. If the recipe is found objMap member will have the key "Recipe" and
    the Mfg0MEMBOMRecipe objects as the values, the strMap will have "Name" for key and the corresponding names of the
    recipes, the objMap will have "Anchor" to indicate the BOMLine object owning the recipe.
    :var serviceData: Service data capturing partial errors using the input array index as client id.
    """
    info: List[AdditionalInfo] = ()
    serviceData: ServiceData = None


@dataclass
class AssignmentRecipeSearchCriteria(TcBaseObj):
    """
    search criteria
    
    :var objectTypes: the types of objects to search.
    :var logicalDesignator: name, value attributes of logicalDesignator
    :var refinements: refinements on the search
    :var absoccAttributes: KeyValue Pairs for each attribute on AbsOccurrence to be used in search.
    """
    objectTypes: List[BusinessObject] = ()
    logicalDesignator: KeyValuePair = None
    refinements: List[MFGSearchRefinementSet] = ()
    absoccAttributes: List[KeyValuePair] = ()


@dataclass
class KeyValuePair(TcBaseObj):
    """
    A generic name/value pair construct.
    
    :var name: name of a key
    :var value: property value.
    """
    name: str = ''
    value: str = ''


@dataclass
class ResolveAssignmentRecipeInputElement(TcBaseObj):
    """
    Element decribing the input for resolveAssignmentRecipe.
    
    :var recipeAnchor: The object on or below which the recipe should be resolved. Currently, only BOMLine objects are
    supported.
    :var recipeObjs: A list of recipe objects that are a subset of recipes attached to recipeAnchor. If this list is
    not empty there is no search performed on recipeAnchor to get any recipe objects for resolve. Currently, this
    object must be of type Mfg0MEMBOMRecipe.
    :var searchScopes: The Engineering BOM (EBOM) scopes to resolve the objects to be assigned to skeletal
    Manufacturing BOM (MBOM). Currently, only the root scope ( EBOM root ) BOMLine is supported.
    :var reResolve: If true force resolve by eliminating previous result. Should be true unless, already resolved lines
    are being asked for.
    :var removePreviouslyResolved: If true, any previously resolved lines are removed.
    :var recursive: If true, recursively resolve under the given recipe anchor
    :var generateLog: If true, generates a log file with details of the resolved lines for given recipe under a parent
    line.
    :var appKey: The application key to decide which resolver to use. Currently, only key supported is
    Mfg0AssignmentRecipe.
    """
    recipeAnchor: BusinessObject = None
    recipeObjs: List[BusinessObject] = ()
    searchScopes: List[BusinessObject] = ()
    reResolve: bool = False
    removePreviouslyResolved: bool = False
    recursive: bool = False
    generateLog: bool = False
    appKey: str = ''


@dataclass
class ResolveAssignmentRecipeResp(TcBaseObj):
    """
    ResolveAssignmentRecipeResp structure contains a list of AdditionalInfo elements, the size of which matches the
    input element list. It also includes the standard serviceData object.
    
    :var info: A list of AdditionInfo structures. The names of the resolved recipes appear under the "Recipe" key of
    strMap. Each value in this strMap is a further key into objMap and the values for those are the resolved BOMLine
    objects. Any BOMLine objects that are manually assigned from EBOM to MBOM appear under the objMap key "Manual",
    with the values being the BOMLine objects that were manually assigned. If "generateLog" is true in the input then a
    file ticket is generated under the "LogFileTicket" key of strMap of the first element in this list.
    :var serviceData: serviceData to caputre partial errors.
    """
    info: List[AdditionalInfo] = ()
    serviceData: ServiceData = None


@dataclass
class CreateOrUpdateAssignmentRecipeInputElem(TcBaseObj):
    """
    An element defining the content and context for attaching the recipe to be used to populating child nodes.
    
    :var name: name of the recipe.
    :var recipeAnchorOrRecipe: The anchor to which the recipe definition will be attached after creation of recipe, or
    the recipe to be updated.
    :var searchContextSC: The object containing the details of search fields supported by cacheless search. Currently,
    this must be a StructureSearchContext with details of cacheless search attributes.
    :var additionalSearchCriteria: additional search criteria with manufacturing aspects.
    :var appKey: A unique key used for choosing the right resolver object. Currently, only supports Mfg0AssignmentRecipe
    """
    name: str = ''
    recipeAnchorOrRecipe: BusinessObject = None
    searchContextSC: BusinessObject = None
    additionalSearchCriteria: AssignmentRecipeSearchCriteria = None
    appKey: str = ''


@dataclass
class CreateOrUpdateAssignmentRecipeResp(TcBaseObj):
    """
    CreateOrUpdateAssignmentRecipeResp structure contains a list of AdditionalInfo elements, the size of which matches
    the input element list. It also includes the standard serviceData object. Each element contains the recipe object (
    stringToObjVector with key "CreatedObject"  or "UpdatedObject" and value being the Mfg0MEMBOMRECIPE). In case there
    is a failure - the element content will be empty - and the serviceData will have the details of the failure for
    that particular element.
    
    
    :var info: A list of AdditionalInfo structures. If recipe is created, the objMap member will have the key
    "CreatedObject" with the value being the Mfg0MEMBOMRecipe object. If recipe is updated the objMap member will have
    the key "UpdatedObject". The strMap member will have the "Name" key with the value being a element of size one
    having the name of the recipe.
    :var serviceData: serviceData
    """
    info: List[AdditionalInfo] = ()
    serviceData: ServiceData = None


@dataclass
class AdditionalInfo(TcBaseObj):
    """
    a generic structure to capture additional information.
    - intMap    A map of string to a list of integers. 
    - dblMap    A map of string to a list of doubles. 
    - strMap    A map of string to a list of strings. 
    - objMap    A map of string to a list of BusinessObjects.
    - dateMap    A map of string to a list of dates.
    
    
    
    :var intMap: A map (string/list of integers) of generic key to integer values.
    :var dblMap: A map (string/list of doubles) of generic key to double values.
    :var strMap: A map (string/list of strings) of generic key to string values.
    :var objMap: A map (string/list of BusinessObjects) of generic key to  BusinessObject values.
    :var dateMap: A map (string/list of dates) of generic key to date values.
    """
    intMap: StringToIntVectorMap = None
    dblMap: StringToDblVectorMap = None
    strMap: StringtoStrVectorMap = None
    objMap: StringToObjVectorMap = None
    dateMap: StringToDateVectorMap = None


"""
a map of string to vector of dates
"""
StringToDateVectorMap = Dict[str, List[datetime]]


"""
String to vector of doubles map.
"""
StringToDblVectorMap = Dict[str, List[float]]


"""
map of string to vector of integers.
"""
StringToIntVectorMap = Dict[str, List[int]]


"""
a map of string to vector of objects.
"""
StringToObjVectorMap = Dict[str, List[BusinessObject]]


"""
A map of string to vector of strings.
"""
StringtoStrVectorMap = Dict[str, List[str]]



from tcsoa.gen.BusinessModeler._2008_06.services import ConditionEngineService as imp0
from tcsoa.gen.BusinessModeler._2007_06.services import RulesBasedFrameworkService as imp1
from tcsoa.gen.BusinessModeler._2008_06.services import DeepCopyRulesService as imp2
from tcsoa.gen.BusinessModeler._2007_06.services import ConstantsService as imp3
from tcsoa.gen.BusinessModeler._2011_06.services import ConstantsService as imp4
from tcsoa.gen.BusinessModeler._2010_04.services import BusinessRulesService as imp5
from tcsoa.utils import TcService


class ConditionEngineService(TcService):
    evaluateConditions = imp0.evaluateConditions


class RulesBasedFrameworkService(TcService):
    executeRbfRules = imp1.executeRbfRules


class DeepCopyRulesService(TcService):
    getDeepCopyInfo = imp2.getDeepCopyInfo


class ConstantsService(TcService):
    getGlobalConstantValues = imp3.getGlobalConstantValues
    getGlobalConstantValues2 = imp4.getGlobalConstantValues2
    getPropertyConstantValues = imp3.getPropertyConstantValues
    getTypeConstantValues = imp3.getTypeConstantValues


class BusinessRulesService(TcService):
    getVerificationRules = imp5.getVerificationRules



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import List
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class RIAttributeInfo(TcBaseObj):
    """
    The 'RIAttributeInfo' structures holds the name/value pairs of import option. Value can be single or multiple
    valued. The import options influences the business object export and has default value.
    
    :var name: Import option name for remote import.
    
    :var value: Values for remote import option (can be multi-valued strings).
    """
    name: str = ''
    value: List[str] = ()


@dataclass
class RemoteImportInfo(TcBaseObj):
    """
    The 'RemoteImportInfo' holds the business object and related information meant for remote import.
    
    
    :var object: Object for the remote import service
    :var reason: User input to explain, why the remote import of business object required. Its not a mandatory input
    and user can pass empty string.
    :var importOptions: Import options that influence, how the Business Objects are imported to target site.
    """
    object: BusinessObject = None
    reason: str = ''
    importOptions: List[RIAttributeInfo] = ()



from __future__ import annotations

from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class ExecuteRuleResponse(TcBaseObj):
    """
    ExecuteRuleResponse represents the outputs of the internal execute StructureMap and execute Data Map operations.
    
    :var serviceData: If the root of the output structure has a BOMView Revision, PSBOMViewRevision of the root of the
    output structure is returned as a part of Created Object in the Service Data, else the ItemRevision of the root is
    returned as a part of the Created Object in the Service Data.
    :var activityLog: A log containing the results of the Data Map and StructureMap rules applied to the input
    structure and the output items created. The details include what Data Map and StructureMap rules were applied, type
    of the output item created, the Item ID of the output item, the relationships created between the input and the
    output ItemRevision. Any failures in creation of the output item or relationships are also returned as a part of
    the activity log.
    
    Following are some possible errors returned in ServiceData:
    - 206622 Structure Engine unable to load/read/parse Data Map.
    - 206642 XML Libraries for StructureMap Engine not available.
    - 206643 CAE_dataMapping_file preference not defined.
    - 206647 Item creation failed, operation aborted.
    - 206648 Occurrence creation failed, operation aborted.
    - 206649 Unknown attribute found.
    - 206650 Object not modifiable, set attribute operation failed.
    - 206651 Form creation failed.
    - 206652 BOMView creation failed.
    - 206653 Unable to save the Item in the Newstuff folder.
    - 206658 Existing Relationship found, relationship not being created.
    - 206662 Error encountered in Variant Condition creation.
    - 206664 Error in relationship creation.
    - 206665 Item node line definition missing in Data Map.
    - 206666 StructureMap domain not found for StructureMapRevision.
    - 206672 Rules with different domains found in the same StructureMap/Data Map.
    - 206673 Rules with no domain defined found in StructureMap/Data Map.
    - 206677 Multiple variant clauses found in the variant condition. Variant Condition creation failed.
    - 206678 Invalid or missing variant clause expression. Variant Condition creation failed.
    - 206679 Mapped BOMView Type does not exist, creating default view type.
    
    """
    serviceData: ServiceData = None
    activityLog: str = ''



from __future__ import annotations

from tcsoa.gen.BusinessObjects import ItemRevision, PSViewType, PSBOMViewRevision, Item
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class GetAllAvailableViewTypesInput(TcBaseObj):
    """
    The 'GetAllAvailableViewTypesInput' denotes input structure for getting available PSBOMView types.
    
    :var clientId: Identifier that helps the client to track the object(s) created.
    :var itemRevisionObj: Refers to ItemRevision object on which we find the available PSBOMView Types.
    :var itemObject: Refers to the Item object of itemRevisionObj.
    """
    clientId: str = ''
    itemRevisionObj: ItemRevision = None
    itemObject: Item = None


@dataclass
class GetAvailableViewTypesOutput(TcBaseObj):
    """
    This contains 'GetAvailableViewTypesOutput' struct which has a list of available PSBOMView types on given
    ItemRevision object.
    
    :var clientId: Identifier that helps the client to track the object(s) created.
    :var viewTags: Refers to a list of PSBOMView types available for given ItemRevision object.
    """
    clientId: str = ''
    viewTags: List[PSViewType] = ()


@dataclass
class GetAvailableViewTypesResponse(TcBaseObj):
    """
    This contains 'GetAvailableViewTypesResponse' object which refers to the available PSBOMView types on given
    ItemRevision object.
    
    :var serviceData: SOA Framework class that holds model objects and partial errors.
    :var viewTypesOutputs: Refers to a list of 'GetAvailableViewTypesOutput' output structure, and has a list of
    PSBOMView types for given ItemRevision object.
    """
    serviceData: ServiceData = None
    viewTypesOutputs: List[GetAvailableViewTypesOutput] = ()


@dataclass
class CreateOrSaveAsPSBOMViewRevisionInput(TcBaseObj):
    """
    This contains inputs related to Item object and ItemRevision object on which we need to create the
    PSBOMViewRevision object.
    
    :var clientId: Identifier that helps the client to track the object(s) created.
    :var itemObject: Item object associated with PSBOMViewRevision object. It is mandatory in case of create and Save
    As scenario.
    :var itemRevObj: Refers to ItemRevision object associated with PSBOMViewRrevision object. It is mandatory in Save
    As operation and optional in case of Create operation. In case of create if this do not present, then the latest
    ItemRevision of itemObject will be used.
    :var viewTypeTag: Refers to PSViewType object in case of save as operation.
    :var srcBvrTag: Refers to source PSBOMViewRevision object. If srcBvrTag is null object then Create operation will
    be performed. If srcBvrTag is not null object, it will be used for Save As operation.
    In case of Save As scenario if srcBvrTag object's PSBOMView and viewTypeTag are same then the new PSBOMViewRevision
    object created as revise operation on given Item object and ItemRevision object. If they are different then the new
    PSBOMViewRevision object created as copy operation on given itemObject and itemRevObj
    
    :var isPrecise: This indicates that PSBOMViewRevision to be created as precise or not in case of creating new
    PSBOMViewRevision for Create Operation. This parameter is not used in the Save As operation.
    """
    clientId: str = ''
    itemObject: Item = None
    itemRevObj: ItemRevision = None
    viewTypeTag: PSViewType = None
    srcBvrTag: PSBOMViewRevision = None
    isPrecise: bool = False


@dataclass
class CreateOrSaveAsPSBOMViewRevisionOutput(TcBaseObj):
    """
    The 'CreateOrSaveAsPSBOMViewRevisionOutput' is output data structure and it contains newly created
    PSBOMViewRevision object.
    
    :var clientId: Identifier that helps the client to track the object(s) created.
    :var bvrTag: This has PSBOMViewRevision object created as a result of create or save as action.
    """
    clientId: str = ''
    bvrTag: PSBOMViewRevision = None


@dataclass
class CreateOrSaveAsPSBOMViewRevisionResponse(TcBaseObj):
    """
    The 'CreateOrSaveAsPSBOMViewRevisionResponse' is response structure for create or save as action for
    PSBOMViewRevision object.
    
    :var serviceData: SOA Framework class that holds model objects and partial errors.
    :var psBVROutputs: This has list of 'CreateOrSaveAsPSBOMViewRevisionOutput' struct output data which contains the
    newly created PSBOMViewRevision object.
    """
    serviceData: ServiceData = None
    psBVROutputs: List[CreateOrSaveAsPSBOMViewRevisionOutput] = ()



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from tcsoa.gen.Manufacturing._2017_05.ImportExport import MfgImportFromBriefcaseResponse, NamesAndValues
from tcsoa.gen.Manufacturing._2017_05.Validation import RegisteredCallbackObjectsResponse
from tcsoa.utils import TcService


class ImportExportService(TcService):

    @classmethod
    def importFromBriefcase(cls, fmsTicket: str, optionSetTag: BusinessObject, optionNamesAndValues: NamesAndValues, sessionOptionAndValues: NamesAndValues) -> MfgImportFromBriefcaseResponse:
        """
        This operation is applicable specifically for Manufacturing Process Planner MPP application.
        This operation performs following operation
        Teamcenter::Soa::GlobalMultiSite::_2008_06::ImportExport importObjectsFromOfflinePackage
        In addition to this, it supports asynchronous import of briefcase.
        
        Use cases:
        Use Case 1: Importing objects from briefcase
        This operation can be used in Manufacturing Process Planner (MPP) application to import Briefcase file into
        Teamcenter. Briefcase file is a zipped file containing TC XML and data set files. The TC XML file specifies the
        object to be imported. The import dialog presents various option sets to control the objects during import.
        First time import will create the objects and upload the datasets. Subsequent import of same object will update
        it. Importer log is generated and presented after the import.
        """
        return cls.execute_soa_method(
            method_name='importFromBriefcase',
            library='Manufacturing',
            service_date='2017_05',
            service_name='ImportExport',
            params={'fmsTicket': fmsTicket, 'optionSetTag': optionSetTag, 'optionNamesAndValues': optionNamesAndValues, 'sessionOptionAndValues': sessionOptionAndValues},
            response_cls=MfgImportFromBriefcaseResponse,
        )


class ValidationService(TcService):

    @classmethod
    def getAllRegisteredCallbacks(cls, callbackType: str) -> RegisteredCallbackObjectsResponse:
        """
        This operation returns all the registered customized callbacks for the given callback  type.
        """
        return cls.execute_soa_method(
            method_name='getAllRegisteredCallbacks',
            library='Manufacturing',
            service_date='2017_05',
            service_name='Validation',
            params={'callbackType': callbackType},
            response_cls=RegisteredCallbackObjectsResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class GetOrResetUIColumnConfigInput(TcBaseObj):
    """
    Contains input information required to retrieve or reset UI column configurations from the Teamcenter database.
    
    :var scope: The scope value is used to apply the UI configurations for that scope. The valid values are :
    "Site" - Returns the configuration defined for the site. 
    "Group" - Returns the configuration defined for a specific Teamcenter Group.
    "Role" - Returns the configuration defined for a specific Teamcenter Role.
    "User" - Returns the configurations defined for a specific Teamcenter User.
    "LoginUser" - Returns the configurations defined for the current login user.
    :var scopeName: The name of a valid Teamcenter scope. Valid scopes are "Site", "Group", "Role", "User",
    "LoginUser". ScopeName should be empty when scope is set for "Site" or "LoginUser". For other scopes use as follows
    
    User    -   User name as per the user's profile
    Group  -   Valid group defined in Organization
    Role    -   Valid role defined in Organization
    :var clientName: The name of a client application, as represented by an instance of Fnd0Client in the Teamcenter
    database. This value must match the value of fnd0ClientName property.
    
    Ex: The client name for Active Workspace is "AWClient".
    :var hostingClientName: Specifies the name of a hosting client application, as represented by an instance of
    Fnd0Client, in the Teamcenter databases. This value must match a value of the fnd0ClientName property. 
    
    For example, if client A is integrated with client B and the user can invoke client B commands from within client
    A, the input to getUiConfigs would specify client A as hosting client and client B as the client. If the caller
    wanted native commands for client A, client A would be specified as client and hosting client would be empty.
    :var resetColumnConfig: If True, it will reset the column configurations at the "LoginUser" scope. Reset action is
    valid only for "LoginUser" scope.
    :var columnConfigQueryInfos: List of column config query information. This has the client scope URI, type names and
    operation type to be used and query for the valid columns.
    :var businessObjects: List of business objects for which the additional properties found as new columns need to be
    loaded.
    """
    scope: str = ''
    scopeName: str = ''
    clientName: str = ''
    hostingClientName: str = ''
    resetColumnConfig: bool = False
    columnConfigQueryInfos: List[ColumnConfigQueryInfo] = ()
    businessObjects: List[BusinessObject] = ()


@dataclass
class GetOrResetUIColumnConfigResponse(TcBaseObj):
    """
    This structure returns information to the client about column configurations. The ServiceData contains information
    about errors encountered during processing.
    
    :var clientName: The client for which the configurations are applicable.
    :var columnConfigurations: List of column configurations data.
    :var serviceData: ServiceData structure containing errors and command, command collection and icon objects. If
    there is an error retrieving the configuration information, the error added to the ServiceData as a partial error.
    """
    clientName: str = ''
    columnConfigurations: List[ColumnConfigData] = ()
    serviceData: ServiceData = None


@dataclass
class GetVisibleCommandsInfo(TcBaseObj):
    """
    Contains input information required to retrieve the commands that are visible from the Teamcenter database.
    
    :var clientScopeURI: The client scope URI represents location.sublocation in Active Workspace. For example,
    Fnd0ClientScope::fnd0ClientScopeURI. If empty, the UI Configuration for all client scopes are returned.
    :var selectionInfo: List of objects selected by the user in the client.
    :var commandInfo: List of commands.
    :var commandContextInfo: List of command context name value pairs, for example: "isHosted"="true",
    "productContext"=productContextUid
    """
    clientScopeURI: str = ''
    selectionInfo: List[SelectionInfo] = ()
    commandInfo: List[CommandInfo] = ()
    commandContextInfo: List[CommandContextInfo] = ()


@dataclass
class GetVisibleCommandsResponse(TcBaseObj):
    """
    This structure returns information to the client about the commands that are visible. The ServiceData contains
    information about errors encountered during processing.
    
    :var serviceData: ServiceData structure containing errors and command, command collection and icon objects. If
    there is an error retrieving the configuration information, the error added to the ServiceData as a partial error.
    :var visibleCommandsInfo: List of visible commands.
    """
    serviceData: ServiceData = None
    visibleCommandsInfo: List[CommandInfo] = ()


@dataclass
class ColumnConfig(TcBaseObj):
    """
    This structure contains information for a column configuration within a client scope URI. It contains a unique
    column config id, a list of column definition information, and the operation type used to finalize the columns.
    
    :var columnConfigId: The unique identifier of the column configuration.
    :var operationType: The operation that was used to finalize the columns to be returned back.
    Valid values are:
    "Intersection", "Union" and "Configured"
    :var columns: Ordered list of column details. The columns will be listed in ascending order of priority.
    """
    columnConfigId: str = ''
    operationType: str = ''
    columns: List[ColumnDefInfo] = ()


@dataclass
class SaveColumnConfigData(TcBaseObj):
    """
    This structure contains column configuration details and a list of column definition information.
    
    :var scope: The scope value is used to apply the UI configurations for that scope. 
    The Valid values are :
    "LoginUser" - Returns the current configuration defined for the current login user. 
    "scopeName" should be empty when scope is set for LoginUser.
    Note: Other scope values "Site", "Group" and "Role" are not supported.
    :var scopeName: The name of a valid Teamcneter scope. For "Site" and "LoginUser" scope, this value should be empty.
    Note: Added this for future. To be used when code starts supporting saving of column configurations for "Site",
    "Group" and "Role" scopes.
    :var clientScopeURI: The unique name of the client scope containing configurations.
    :var columnConfigId: The unique identifier for the column configuration.
    :var columns: Ordered list of column information.
    """
    scope: str = ''
    scopeName: str = ''
    clientScopeURI: str = ''
    columnConfigId: str = ''
    columns: List[ColumnDefInfo] = ()


@dataclass
class SelectionInfo(TcBaseObj):
    """
    This structure contains the list of objects selected by the user in the client.
    
    :var selectedObjects: The objects selected by the user in the client.
    :var parentSelectionIndex: Parent selection for the object. For example, for cut operation, it is the selected
    parent object.
    :var contextName: Optional context name, for example, for cut operation, it is the relation name between parent
    object and child object.
    """
    selectedObjects: List[BusinessObject] = ()
    parentSelectionIndex: int = 0
    contextName: str = ''


@dataclass
class ColumnConfigData(TcBaseObj):
    """
    This structure returns information about the column configuration definitions for a scope, hosting client and
    client scope.
    
    :var scope: The scope that is used to retrieve the UI configurations.
    :var scopeName: The scope name for which the column data is applicable.
    :var hostingClient: The name of hosting client for which the list of column configurations is applicable. This
    value must correspond to the value of the property fnd0ClientName for an Fnd0Client object in the Teamcenter
    database.
    :var clientScopeURI: The client scope for which the list of column configurations is applicable. This must match a
    value of fnd0ClientScopeURI which is the unique identifier of a client scope (Fnd0ClientScope).
    :var columnConfigurations: List of column configuration details.
    """
    scope: str = ''
    scopeName: str = ''
    hostingClient: str = ''
    clientScopeURI: str = ''
    columnConfigurations: List[ColumnConfig] = ()


@dataclass
class ColumnConfigQueryInfo(TcBaseObj):
    """
    This structure contains the client scope URI to be used and query for its associated column configurations. The
    type names and operation type are used to identify the final list of columns to be returned back.
    
    :var clientScopeURI: The unique name of the client scope containing column configurations.
    Ex: "Awp0SearchResults" for global search.
    :var operationType: The operation that needs to be applied to finalize the columns to be returned back.
    Valid values are:
    "Intersection" - Gets the intersection of the columns for the types passed in.
    "Union" - Gets all the columns of the types passed in.
    "Configured" - Gets all the columns defined for requested scope irrespective of types passed in. If it does not
    find any configuration at the specified scope it will search up in the order of scopes User, Role, Group and then
    Site. No need to pass type names if need to fetch all configured columns.
    :var typeNames: List of type names(internal names) for which the columns needs to be fetched. In search case these
    will the type names of the results found.
    :var columnsToExclude: List of columns which should be excluded from the final list being returned. The value
    provided should be in the format "typeName.propertyName" . Both type name and property name should be internal
    values. 
    Ex: ItemRevision.sequence_id , where '.' Is the delimiter
    """
    clientScopeURI: str = ''
    operationType: str = ''
    typeNames: List[str] = ()
    columnsToExclude: List[str] = ()


@dataclass
class ColumnDefInfo(TcBaseObj):
    """
    Contains details about a specific column. This includes the type of object for which the column is applicable, the
    name of the property displayed in the column, a flag indicating if the column should be used to order information
    displayed in the client, pixel width of the column, a flag indicating if the column should be hidden and the column
    sort priority and direction.
    
    :var typeName: The business object type for the value displayed in the column.
    :var propertyName: The name of the property corresponding to the values displayed in the column.
    :var pixelWidth: The pixel width for the column. Valid pixel widths are integer values between 1 and 500.
    :var columnOrder: The column order value is used to arrange the columns in order.
    :var hiddenFlag: True if the column should be hidden on the client user interface.
    :var sortPriority: Sort priority set on column helps identify the order in which the columns should be used during
    sort. Sort priority value will be zero for columns not marked for sorting.
    :var sortDirection: How the columns are sorted. Valid values are: "Ascending" and "Descending". This value will be
    empty if the column is not marked for sorting.
    """
    typeName: str = ''
    propertyName: str = ''
    pixelWidth: int = 0
    columnOrder: int = 0
    hiddenFlag: bool = False
    sortPriority: int = 0
    sortDirection: str = ''


@dataclass
class CommandContextInfo(TcBaseObj):
    """
    This structure contains the command context name and value
    
    :var contextName: Context Name which is to be used to evaluate visibility of the command
    :var contextValue: Context value that forms a valid pair with the contextName.
    """
    contextName: str = ''
    contextValue: str = ''


@dataclass
class CommandInfo(TcBaseObj):
    """
    This structure contains the Ids of command and command collection that are visible for the indicated client.
    
    :var commandCollectionId: The command collection Id.
    :var commandId: The command Id.
    """
    commandCollectionId: str = ''
    commandId: str = ''



from __future__ import annotations

from tcsoa.gen.BusinessObjects import Dma1PrintConfigRuntime
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class PrinterDefinitionResponse(TcBaseObj):
    """
    Printer Definition Response structure.
    
    :var printConfigurations: A list of Print Configuration Runtime business objects.  This business object contains
    all print configuration information properties such as Print Configuration Name, Printer Name, Paper Sizes, etc.
    :var serviceData: Standard return; includes any error information.
    """
    printConfigurations: List[Dma1PrintConfigRuntime] = ()
    serviceData: ServiceData = None



from __future__ import annotations

from tcsoa.gen.BusinessObjects import GSIdentity
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class GetHashedUIDResponse(TcBaseObj):
    """
    The GetHashedUIDResponse structure defines the response from
    getHashedUID operation. It contains hashed uid generated by taking the hash key and site-id as input.
    
    :var hashedUID: The hashed uid  generated.
    :var serviceData: Service data contains the list of created or modified objects and also the partial errors.
    """
    hashedUID: str = ''
    serviceData: ServiceData = None


@dataclass
class TIEGSIdentityInput(TcBaseObj):
    """
    The GS identity input structure contains the fields required to create a GS identity object.
    
    :var system: Attribute system of class system GSIdentity.This is the site id of source system.
    :var label: Attribute label of class GSIdentity
    :var subLabel: Attribute subLabel of of class GSIdentity
    :var className: Attribute className of class GSIdentity.This is the className of the Teamcenter BusinessObject for
    which GSId is being created.
    :var splitToken: Attribute split_token of class GSIdentity
    :var context: Attribute context of class GSIdentity
    :var objectUid: Attribute object_uid of class GSIdentity.This is 14 digit unique identifier.
    :var factorId: Attribute factor_id of class GSIdentity
    :var atomic: Attribute atomic of class GSIdentity
    """
    system: str = ''
    label: str = ''
    subLabel: str = ''
    className: str = ''
    splitToken: str = ''
    context: str = ''
    objectUid: str = ''
    factorId: str = ''
    atomic: bool = False


@dataclass
class CreateGSIdentitiesResponse(TcBaseObj):
    """
    The CreateGSIdentitiesResponse structure defines the response from
    createGSIdentities operation.It contains vector of GS identity object
    references that were created.
    
    :var gsIdentities: A list of GS identity object references.
    :var serviceData: Service data contains the list of created or modified objects and also the partial errors.
    """
    gsIdentities: List[GSIdentity] = ()
    serviceData: ServiceData = None



from __future__ import annotations

from typing import List
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class ServerCacheResponse(TcBaseObj):
    """
    ServerCacheResponse structure represents the error code that could occur during the creation or deletion of the
    Metadata cache and corresponding log file tickets.
    
    :var errorCode: Error code for the cache creation process. It is the error code returned by the server if there is
    any error during cache generation or deletion.
    :var logFileTickets: Log file tickets corresponding to each log file generated. These are the FMS file tickets that
    BMIDE uses to retrieve the log files to be displayed to the BMIDE user.
    """
    errorCode: int = 0
    logFileTickets: List[str] = ()



from __future__ import annotations

from tcsoa.gen.Internal.Administration._2013_05.UserManagement import LicenseStatusResponse, ActivateUserInput
from typing import List
from tcsoa.utils import TcService


class UserManagementService(TcService):

    @classmethod
    def activateUsers2(cls, activateUsers: List[ActivateUserInput]) -> LicenseStatusResponse:
        """
        This operation activates user(s) based on the number of allowed active author, consumer, or occasional user
        licenses,  associated with license bundles. If enough licenses are not available for either the license level
        or the license bundle, a partial error is returned with the index of the activateUsers input.
        """
        return cls.execute_soa_method(
            method_name='activateUsers2',
            library='Internal-Administration',
            service_date='2013_05',
            service_name='UserManagement',
            params={'activateUsers': activateUsers},
            response_cls=LicenseStatusResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import List
from tcsoa.gen.Multisite._2019_06.ImportExportTCXML import RemoteCheckinResponse, RemoteCheckoutResponse, StringVectorMap
from tcsoa.gen.Multisite._2019_06.Search import StringVectorMap1, SearchResponse
from tcsoa.utils import TcService


class SearchService(TcService):

    @classmethod
    def publish(cls, objects: List[BusinessObject], targetSites: List[str], options: StringVectorMap1) -> SearchResponse:
        """
        The publish operation applies the Multi-Site publish to the Object Directory Server (ODS) operation to the
        business objects in the objects parameter.
        
        Use cases:
        In the Multi-Site federation, a user can share the business objects among the participating sites, the client
        shares business object(s) using the remote import or remote export functionality. Sites can publish objects to
        the ODS which allows a remote site to search for candidates to remote import.
        """
        return cls.execute_soa_method(
            method_name='publish',
            library='Multisite',
            service_date='2019_06',
            service_name='Search',
            params={'objects': objects, 'targetSites': targetSites, 'options': options},
            response_cls=SearchResponse,
        )

    @classmethod
    def unpublish(cls, objects: List[BusinessObject], targetSites: List[str], options: StringVectorMap1) -> SearchResponse:
        """
        The unpublish operation applies the Multi-Site unpublish to the Object Directory Server (ODS) operation to the
        business objects in the objects input list.
        
        Use cases:
        In the Multi-Site federation, a user can share the business objects among the participating sites, the client
        shares business object(s) using the remote import or remote export functionality. Sites can publish objects to
        the ODS which allows a remote site to search for candidates to remote import. These same objects can be later
        unpublished.
        """
        return cls.execute_soa_method(
            method_name='unpublish',
            library='Multisite',
            service_date='2019_06',
            service_name='Search',
            params={'objects': objects, 'targetSites': targetSites, 'options': options},
            response_cls=SearchResponse,
        )


class ImportExportTCXMLService(TcService):

    @classmethod
    def remoteCheckin(cls, objects: List[BusinessObject], optionSetName: str, sessionOptions: StringVectorMap, overrideOptions: StringVectorMap) -> RemoteCheckinResponse:
        """
        The remoteCheckin operation applies the Multi-Site checkin operation to the business objects in the objects
        parameter. The Multi-Site TCXML payload is used.
        
        Use cases:
        In the Multi-Site federation, a user can share the business objects among the participating sites, the client
        shares business object(s) using the remote import or remote export functionality. This allows the creation of
        replica objects which are read-only copies of the master object. Multi-site supports a Remote Check-in
        Check-out feature which allows edits to replica objects. This operation supports the following use case. 
        
        * The remote checkin of checked-out replica business objects.  Changes made to the replica object will be sent
        back to the master site and the checked-out status will be removed at both the master and replica objects.
        """
        return cls.execute_soa_method(
            method_name='remoteCheckin',
            library='Multisite',
            service_date='2019_06',
            service_name='ImportExportTCXML',
            params={'objects': objects, 'optionSetName': optionSetName, 'sessionOptions': sessionOptions, 'overrideOptions': overrideOptions},
            response_cls=RemoteCheckinResponse,
        )

    @classmethod
    def remoteCheckout(cls, objects: List[BusinessObject], optionSetName: str, sessionOptions: StringVectorMap, overrideOptions: StringVectorMap) -> RemoteCheckoutResponse:
        """
        The RemoteCheckout operation applies the Multi-Site check out operation to the business objects in the objects
        parameter.
        
        Use cases:
        In the Multi-Site federation, a user can share the business objects among the participating sites, the client
        shares business object(s) using the remote import or remote export functionality. This allows the creation of
        replica objects which are read-only copies of the master object. Multi-site supports a Remote Check-in
        Check-out feature which allows edits to replica objects. This operation supports the following use case.
        
        - The remote check out of replica business objects.  This allows edits to the replica objects. These changes
        are later saved to the owning site with the remote check in operation.
        
        """
        return cls.execute_soa_method(
            method_name='remoteCheckout',
            library='Multisite',
            service_date='2019_06',
            service_name='ImportExportTCXML',
            params={'objects': objects, 'optionSetName': optionSetName, 'sessionOptions': sessionOptions, 'overrideOptions': overrideOptions},
            response_cls=RemoteCheckoutResponse,
        )



from __future__ import annotations

from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.gen.BusinessObjects import TC_Project
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class LoadProjectDataForUserResponse(TcBaseObj):
    """
    Object that holds applicable projects.
    
    :var applicableProjects: List of TC_project objects found.
    :var serviceData: A  standard  ServicData.
    """
    applicableProjects: List[TC_Project] = ()
    serviceData: ServiceData = None



from tcsoa.gen.ChangeManagement._2008_06.services import ChangeManagementService as imp0
from tcsoa.gen.ChangeManagement._2015_10.services import ChangeManagementService as imp1
from tcsoa.gen.ChangeManagement._2020_01.services import ChangeManagementService as imp2
from tcsoa.gen.ChangeManagement._2009_06.services import ChangeManagementService as imp3
from tcsoa.utils import TcService


class ChangeManagementService(TcService):
    configureChangeSearches = imp0.configureChangeSearches
    createBOMEdits = imp0.createBOMEdits
    createChangeItems = imp0.createChangeItems
    createChangeLineage = imp1.createChangeLineage
    createSupercedures = imp0.createSupercedures
    deleteChangeLineage = imp1.deleteChangeLineage
    deleteSupercedures = imp0.deleteSupercedures
    deriveChangeItems = imp2.deriveChangeItems
    findContextData = imp0.findContextData
    findSupersedure = imp0.findSupersedure
    getAllChangeHomeFolders = imp0.getAllChangeHomeFolders
    getBOMEdits = imp0.getBOMEdits
    getNoteVariantChanges = imp3.getNoteVariantChanges
    updateChangeItems = imp0.updateChangeItems
    updateSupercedures = imp0.updateSupercedures
